package sjl;

/**
 * The <b>Algo</b> class contains static methods that implements the generic
 * algorithms.
 * <p>
 * Certain shorthand notaions are throughout the documentation of the
 * algorithm functions.
 * <ul>
 *   <li><code>N</code> is the number of elements in the range 
 *                      <code>[first, last)</code>.
 *   <li><code>N1</code> is the number of elements in the range 
 *                      <code>[first1, last1)</code>.
 *   <li><code>N2</code> is the number of elements in the range 
 *                      <code>[first2, last2)</code>.
 *   <li><code>func(...)</code> means the function object <code>func</code>
 *                      is performed.
 *   <li><code>pred(...)</code> means the predicate object <code>pred</code>
 *                      is tested.
 *   <li><code>...</code> in a range means that the end of the range
 *                      depends on some other parameters, 
 *                      typically <code>N</code>.
 * </ul>
 *
 * <p>Copyright <a href=..\Copyrght.htm>&copy</a> 1996 Finn Bock<p>
 */

public class Algo {

    /**
     * The maximum size of temporary buffers. Temporary buffer are used
     * while sorting.
     */
    public static int tmp_buffer_size = 4096;

    /**
     * Used in quick sort.
     */
    public static int sortThreshold = 16;

    /**
     * Used in merge sort.
     */
    public static int sortChunkSize = 16;

    /**
     * We do not allow instances of the Algo class
     */
    private Algo() {
    }

    /**
     * Advance the iterator a number of times.
     * The iterator <code>i</code> is advanced <code>distance</code>
     * times by calling <code>next()</code>.
     * @param i The iterator to be moved forward.
     * @param distance The number of times the iterator must be moved.
     */
    public static void advance(InputIterator i, int distance) {
	while (distance-- > 0)
	    i.next();
    }

    /**
     * Advance the iterator a number of times.
     * The iterator <code>i</code> is advanced <code>distance</code>
     * times by calling <code>next()</code>.
     * <code>distance</code> can be negative, is this case the iterator
     * is moved backwards by calling <code>prev()</code>.
     *
     * @param i The iterator to be moved forward or backward.
     * @param distance The number of times the iterator must be moved.
     */
    public static void advance(BidirectionalIterator i, int distance) {
	if (distance >= 0)
	    while (distance-- > 0)
		i.next();
	else
	    while (distance-- > 0)
		i.prev();
    }

    /**
     * Advance the iterator a number of times.
     * The iterator <code>i</code> is advanced <code>distance</code>
     * position by calling <code>next(distance)</code>.
     * @param i The iterator to be moved forward or backward.
     * @param distance The number of position the iterator must be moved.
     */
    public static void advance(RandomIterator i, int distance) {
	i.next(distance);
    }
   
    /**
     * Apply a function object to each element in the range.
     * <code>for_each</code> applies <code>func</code> to each element
     * in the range <code>[first, last)</code>. 
     * <code>func</code> is assumed not to change the elements.
     * <code>func</code> is applied exactly <code>N</code> times.
     * If <code>func</code> returns a result, the result is ignored.
     * @param first The beginning of the range
     * @param last The end of the range.
     * @param func the function object that is perform.
     * @return func 
     */
    public static Function1 for_each(InputIterator first, InputIterator last, Function1 func) {
	first = Iter.copy(first);
	while (!first.cmp(last)) {
	    func.perform(first.get());
	    first.next();
	}
	return func;
    }

    /**
     * Find the first elements matching a specified value.
     * Find returns the first iterator <code>i</code> in the range 
     * <code>[first,last)</code> for which the following condition 
     * holds: <code>i.get().equals(value)</code>.
     * If the no such iterator is found, <code>last</code> is returned.
     * @param first The beginning of the range
     * @param last The end of the range.
     * @param value that must match.
     * @return the iterator pointing to the found element.
     */
    public static InputIterator find(InputIterator first, InputIterator last, Object value) {
	first = Iter.copy(first);
	while (!first.cmp(last) && !first.get().equals(value))
	    first.next();
	return first;
    }

    /**
     * Find the first element satisfying a predicate.
     * Find returns the first iterator <code>i</code> in the range 
     * <code>[first, last)</code> for which the following condition 
     * holds: <code>pred(i.get(), value)</code>.
     * If the no such iterator is found, <code>last</code> is returned.
     * @param first The beginning of the range
     * @param last The end of the range.
     * @param pred the predicate that must be true,
     * @return the iterator pointing to the found element.
     */
    public static InputIterator find(InputIterator first, InputIterator last, Predicate1 pred) {
	first = Iter.copy(first);
	while (!first.cmp(last) && !pred.compare(first.get()))
	    first.next();
	return first;
    }

    /**
     * Find the first instance of two equal adjacent elements.
     * <code>adjacent_find</code> returns the first iterator <code>i</code> 
     * in the range <code>[first,last)</code> where 
     * <code>i.get().equal(i.next().get()) == true</code>
     * If no such iterator is found, <code>last</code> is returned.
     * @param first The beginning of the range.
     * @param last The end of the range.
     * @return the iterator pointing to the found element.
     */
    public static ForwardIterator adjacent_find(ForwardIterator first, ForwardIterator last) {
	first = Iter.copy(first);
	if (first.cmp(last)) 
	    return last;
	ForwardIterator next = Iter.copy(first);
	while (!next.next().cmp(last)) {
	    if (first.get().equals(next.get()))
		return first;
	    first.next();
	}
	return last;
    }

    /**
     * Find the first instance of two adjacent elements 
     * satisfying a predicate.
     * <code>adjacent_find</code> returns the first iterator in the range 
     * <code>[first,last)</code> where 
     * <code>pred(i.get(), i.next().get()) == true</code>.
     * If no such iterator is found, <code>last</code> is returned.
     * @param first The beginning of the range
     * @param last The end of the range.
     * @param pred the predicate that must be true.
     * @return the iterator pointing to the found element.
     */
    public static ForwardIterator adjacent_find(ForwardIterator first, ForwardIterator last, Predicate2 pred) {
	first = Iter.copy(first);
	if (first.cmp(last)) 
	    return last;
	ForwardIterator next = Iter.copy(first);
	while (!next.next().cmp(last)) {
	    if (pred.compare(first.get(), next.get()))
		return first;
	    first.next();
	}
	return last;
    }

    /**
     * Return the number of elements in the range equal to a value.
     * <code>count</code> returns the number of iterator <code>i</code> 
     * in the range <code>[first,last)</code> where 
     * <code>i.get().equals(value) == true</code>. 
     * Note that this differs from STL. 
     * @param first The beginning of the range
     * @param last The end of the range.
     * @param value the object which is tested against.
     * @return the number of elements matching value.
     */
    public static int count(InputIterator first, InputIterator last, Object value) {
	first = Iter.copy(first);
	int n = 0;
	while (!first.cmp(last)) {
	    if (first.get().equals(value))
		n++;
	    first.next();
	}
	return n;
    }

    /**
     * Return the number of element satisfying a predicate.
     * <code>count</code> returns the number of iterators <code>i</code> 
     * in the range </code>[first,last)</code> where 
     * </code>pred(i.get()) == true</code>.
     * Note that this differs from STL. 
     * @param first The beginning of the range
     * @param last The end of the range.
     * @param pred the predicate that must be true.
     * @return the number of elements for which pred returned true.
     */
    public static int count(InputIterator first, InputIterator last, Predicate1 pred) {
	first = Iter.copy(first);
	int n = 0;
	while (!first.cmp(last)) {
	    if (pred.compare(first.get()))
		n++;
	    first.next();
	}
	return n;
    }

    /**
     * Find the first instance of mismatch between two ranges.
     * <code>mismatch</code> returns the pair of iterators 
     * <code>i</code> and <code>j</code> such that <code>j</code> and 
     * <code>i</code> is the first iterators in the range 
     * <code>[first1, last1)</code> for which 
     * <code>i.get().equals(j.get()) == false</code>.
     * If such iterators is not found, <code>last1</code> and the corresponding 
     * iterator from <code>first2</code> is returned.
     * @param first1 The beginning of the range
     * @param last1 The end of the range.
     * @param first2 The beginning of the second range
     * @return a pair of iterators pointing to first elements not matching.
     */
    public static Pair mismatch(InputIterator first1, InputIterator last1, InputIterator first2) {
	first1 = Iter.copy(first1);
	first2 = Iter.copy(first2);
	while (!first1.cmp(last1) && first1.get().equals(first2.get())) {
	    first1.next();
	    first2.next();
	}
	return new Pair(first1, first2);
    }

    /**
     * Find the first instance of mismatch between two ranges when
     * comparing with a predicate.
     * <code>mismatch</code> returns the pair of iterators <code>i</code> 
     * and <code>j</code> such that <code>j</code> and <code>i</code> is 
     * the first iterators in the range <code>[first1,last1)</code>
     * for which <code>pred(i.get(), j.get()) == false</code>.
     * If such iterators is not found, <code>last1</code> and the corresponding 
     * iterator from <code>first2</code> is returned.
     * @param first1 The beginning of the range
     * @param last1 The end of the range.
     * @param first2 The beginning of the second range
     * @param pred The predicate used when comparing.
     * @return a pair of iterators pointing to first elements not matching.
     */
    public static Pair mismatch(InputIterator first1, InputIterator last1, InputIterator first2, Predicate2 pred) {
	first1 = Iter.copy(first1);
	first2 = Iter.copy(first2);
	while (!first1.cmp(last1) && pred.compare(first1.get(), first2.get())) {
	    first1.next();
	    first2.next();
	}
	return new Pair(first1, first2);
    }

    /** 
     * Compares to ranges.
     * <code>equal</code> returns <code>true</code> if every element in the 
     * range <code>[first1,last1)</code> is equal 
     * to the corresponding element in the second range.
     * @param first1 The beginning of the range
     * @param last1 The end of the range.
     * @param first2 The beginning of the second range
     * @return A boolean indication whether the ranges are equal.
     */
    public static boolean equal(InputIterator first1, InputIterator last1, InputIterator first2) {
	return ((Iterator)mismatch(first1, last1, first2).first).cmp(last1);
    }

    /**
     * Compares to ranges using a predicate.
     * <code>equal</code> returns <code>true</code> if every element in the 
     * range <code>[first1,last1)</code> is equal 
     * to the corresponding element in the other range.
     * @param first1 The beginning of the range
     * @param last1 The end of the range.
     * @param first2 The beginning of the second range
     * @param pred The predicate used when comparing.
     * @return A boolean indication whether the ranges are equal.
     */
    public static boolean equal(InputIterator first1, InputIterator last1, InputIterator first2, Predicate2 pred) {
	return ((Iterator)mismatch(first1, last1, first2, pred).first).cmp(last1);
    }

    /**
     * Find the first instance in a range where of a subsequence occur.
     * <code>search</code>Finds a subsequence of equal values in a sequence.
     * Returns the first iterator <code>i</code> in the range 
     * <code>[first1,last1)</code> where the subsequence 
     * <code>[first2,last2)</code> is found. 
     * If no such subsequence is found, <code>last1</code> is returned.
     * @param first1 The beginning of the range
     * @param last1 The end of the range.
     * @param first2 The beginning of the second range
     * @param last2 The end of the second range
     * @return A iterator pointer to the start of subsequence found.
     */
    public static ForwardIterator search(ForwardIterator first1, ForwardIterator last1, ForwardIterator first2, ForwardIterator last2) {
	first1 = Iter.copy(first1);
	int d1 = first1.distance(last1);
	int d2 = first2.distance(last2);
	if (d1 < d2) 
	    return last1;
	ForwardIterator current1 = Iter.copy(first1);
	ForwardIterator current2 = Iter.copy(first2);

	while (!current2.cmp(last2)) {
	    if (!current1.get().equals(current2.get())) {
		current1.next();
		current2.next();
		if (d1-- == d2)
		    return last1;
		else {
		    first1.next();
		    current1 = Iter.copy(first1);
		    current2 = Iter.copy(first2);
		}
	    } else {
		current1.next();
		current2.next();
	    }
	}
	return (current2.cmp(last2)) ? first1 : last1;
    }

    /**
     * Find the first instance in a range where of a subsequence occur
     * when comparing using a predicate.
     * <code>search</code> finds a subsequence of equal values in a sequence.
     * Returns the first iterator <code>i</code> in the range 
     * <code>[first1,last1)</code> where the subsequence 
     * <code>[first2,last2)</code> is found. 
     * If no such subsequence is found, <code>last1</code> is returned.
     * @param first1 The beginning of the range
     * @param last1 The end of the range.
     * @param first2 The beginning of the second range
     * @param last2 The end of the second range
     * @param pred The predicate used for comparing elements in the ranges.
     * @return A iterator pointer to the start of subsequence found.
     */
    public static ForwardIterator search(ForwardIterator first1, ForwardIterator last1, ForwardIterator first2, ForwardIterator last2, Predicate2 pred) {
	first1 = Iter.copy(first1);
	int d1 = first1.distance(last1);
	int d2 = first2.distance(last2);
	if (d1 < d2) 
	    return last1;
	ForwardIterator current1 = Iter.copy(first1);
	ForwardIterator current2 = Iter.copy(first2);

	while (!current2.cmp(last2)) {
	    if (!pred.compare(current1.get(), current2.get())) {
		current1.next();
		current2.next();
		if (d1-- == d2)
		    return last1;
		else {
		    first1.next();
		    current1 = Iter.copy(first1);
		    current2 = Iter.copy(first2);
		}
	    } else {
		current1.next();
		current2.next();
	    }
	}
	return (current2.cmp(last2)) ? first1 : last1;
    }

    /**
     * Copy elements from one range to another.
     * <code>copy</code> the elements from the range <code>[first, last)</code>
     * to result.
     * The result of copy is undefined if result is in the range
     * <code>[first,last)</code>.
     * @param first The beginning of the range
     * @param last The end of the range.
     * @param result The destination where the element are copied to.
     * @return A iterator pointer to past-the-end of the result sequence.
     */
    public static OutputIterator copy(InputIterator first, InputIterator last, OutputIterator result) {
	first = Iter.copy(first);
	result = Iter.copy(result);
	while (!first.cmp(last)) {
	    result.put(first.get());
	    result.next();
	    first.next();
	}
	return result;
    }

    /**
     * Copy elements from one range to another, backwards.
     * <code>copy_backward</code> copies the elements from the range 
     * <code>[first, last)</code> to result starting from 
     * <code>last.prev()</code> and proceeding to <code>first</code>. 
     * It should be used instead of <code>copy()</code> when <code>last</code>
     * is in the range <code>[result-N,result), result)</code>.
     * The result of <code>copy_backward</code> is undefined if result is 
     * in the range <code>[first, last)</code>.
     * @param first The beginning of the range
     * @param last The end of the range.
     * @param result The end of the result range.
     * @return A iterator pointer to beginning of the result sequence.
     */
    public static BidirectionalIterator copy_backward(BidirectionalIterator first, BidirectionalIterator last, BidirectionalIterator result) {
	last = Iter.copy(last);
	result = Iter.copy(result);
	while (!first.cmp(last)) {
	    result.prev();
	    last.prev();
	    result.put(last.get());
	}
	return result;
    }

    /**
     * Swap the elements that the iterators points to.
     */
    public static void iter_swap(ForwardIterator a, ForwardIterator b) {
	Object tmp = a.get();
	a.put(b.get());	
	b.put(tmp);
    }

    /**
     * Swap the elements of two ranges.
     * <code>swap_ranges</code> swaps each element in the ranges 
     * <code>[first1,last1)  and [first2,...)</code>.
     * The result of <code>swap_ranges</code> is undefined if the two 
     * ranges overlap.
     * @param first1 The beginning of the range
     * @param last1 The end of the range.
     * @param first2 The beginning of the second range
     * @return A iterator pointing past-the-end of the second range.
     */
    public static ForwardIterator swap_ranges(ForwardIterator first1, ForwardIterator last1, ForwardIterator first2) {
	first1 = Iter.copy(first1);
	first2 = Iter.copy(first2);
	while (!first1.cmp(last1)) {
	    iter_swap(first1, first2);
	    first1.next();
	    first2.next();
	}
	return first2;
    }

    /**
     * Execute a function object on every element in a range, storing
     * the result into another (or the same) range.
     * <code>transform</code> assigns through every iterator in the 
     * range <code>[result,...)</code>
     * a new corresponding value equal to <code>func(i.get())</code> for 
     * each <code>i</code> in the range <code>[first,last)</code>.
     * @param first The beginning of the range
     * @param last The end of the range.
     * @param result The beginning of the result range
     * @return An iterator pointing past-the-end of the result range.
     */
    public static OutputIterator transform(InputIterator first, InputIterator last, OutputIterator result, Function1 func) {
	first = Iter.copy(first);
	result = Iter.copy(result);
	while (!first.cmp(last)) {
	    result.put(func.perform(first.get()));
	    result.next();
	    first.next();
	}
	return result;
    }

    /**
     * Execute a function object on every two elements in two ranges, storing
     * the result into a third (or the same) range.
     * <code>transform</code> assigns through every iterator in the 
     * range <code>[result,...)</code> a new corresponding value equal to
     * <code>func(i1.get(), i2.get())</code> for each <code>i1</code> 
     * and <code>i2</code> in the range <code>[first1,last1)</code>
     * and <code>[first2,...)</code>.
     * @param first1 The beginning of the range
     * @param last1 The end of the range.
     * @param first2 The beginning of the second range
     * @param result The beginning of the result range
     * @return An iterator pointing past-the-end of the result range.
     */
    public static OutputIterator transform(InputIterator first1, InputIterator last1, InputIterator first2, OutputIterator result, Function2 func) {
	first1 = Iter.copy(first1);
	first2 = Iter.copy(first2);
	result = Iter.copy(result);
	while (!first1.cmp(last1)) {
	    result.put(func.perform(first1.get(), first2.get()));
	    result.next();
	    first1.next();
	    first2.next();
	}
	return result;
    }

    /**
     * Replace all elements in the range which match a value, 
     * with another value.
     * <code>replace</code> substitutes elements in the range 
     * <code>[first,last)</code> with <code>value</code> when 
     * <code>i.get().equals(oldvalue) == true</code>.
     * All other elements remains unaffected.
     * @param first The beginning of the range
     * @param last The end of the range.
     * @param oldvalue The elements to be replaced must match oldvalue.
     * @param value The replacement value.
     */
    public static void replace(ForwardIterator first, ForwardIterator last, Object oldvalue, Object value) {
	first = Iter.copy(first);
	while (!first.cmp(last)) {
	    if (oldvalue.equals(first.get())) {
		first.put(value);
	    }
	    first.next();
	}
    }

    /**
     * Replace all element in the range which satisfy a predicate with
     * another value.
     * <code>replace</code> substitutes elements in the range 
     * <code>[first,last)</code> with <code>value</code> when 
     * <code>pred(i.get()) == true</code>.
     * All other elements remains unaffected.
     * @param pred the predicate that must be true for the element 
     *       to be replaced.
     * @param value the replacement value.
     * @param first The beginning of the range
     * @param last The end of the range.
     * @param pred The rpedicate that must be satisfied.
     * @param value The replacement value.
     */
    public static void replace(ForwardIterator first, ForwardIterator last, Predicate1 pred, Object value) {
	first = Iter.copy(first);
	while (!first.cmp(last)) {
	    if (pred.compare(first.get())) {
		first.put(value);
	    }
	    first.next();
	}
    }

    /**
     * Assign a value to all elements in the range.
     * <code>fill</code> fills the range <code>[first,last)</code>
     * with the <code>value</code>. 
     * Note that all elements will share a reference to the same value.
     * @param first The beginning of the range
     * @param last The end of the range.
     * @param value the value that is inserted into the range.
     */
    public static void fill(ForwardIterator first, ForwardIterator last, Object value) {
	first = Iter.copy(first);
	while (!first.cmp(last)) {
	    first.put(value);
	    first.next();
	}
    }

    /**
     * Assign a value to all elements in the range.
     * <code>fill</code> fills the range <code>[first,first+n)</code>
     * with the <code>value</code>. 
     * Note that all elements will share a reference to the same value.
     * @param first The beginning of the range
     * @param n The number of times the value will be inserted.
     * @param value The value that is inserted into the range.
     * @return An iterator pointing past-the-end of the range.
     */
    public static OutputIterator fill(OutputIterator first, int n, Object value) {
	first = Iter.copy(first);
	while (n-- > 0) {
	    first.put(value);
	    first.next();
	}
	return first;
    }

    /**
     * Assign the result of a function object to all elements in the range.
     * <code>generate</code> invokes the function object <code>func</code>
     * and assigns the return value of <code>func</code> through all the 
     * iterators in the range <code>[first,last)</code>. 
     * <code>func</code> takes no arguments.
     * @param first The beginning of the range
     * @param last The end of the range.
     * @param func The function object performed one time for 
     *             each element inserted.
     */
    public static void generate(ForwardIterator first, ForwardIterator last, Function0 func) {
	first = Iter.copy(first);
	while (!first.cmp(last)) {
	    first.put(func.perform());
	    first.next();
	}
    }

    /**
     * Assign the result of a function object to all elements in the range.
     * <code>generate</code> invokes the function object <code>func</code> 
     * and assigns the return value of <code>func</code> through all the 
     * iterators in the range <code>[first, first+n)</code>. 
     * <code>func</code> takes no arguments.
     * @param first The beginning of the range
     * @param n The number of times func will be executed.
     * @param func The function object performed one time for 
     *             each element inserted.
     * @return An iterator pointing past-the-end of the range.
     */
    public static OutputIterator generate(OutputIterator first, int n, Function0 func) {
	first = Iter.copy(first);
	while (n-- > 0) {
	    first.put(func.perform());
	    first.next();
	}
	return first;
    }

    /**
     * Remove all elements in the range which match a value.
     * <code>remove</code> eliminates all the elements referred to by 
     * iterator <code>i</code> in the range <code>[first,last)</code> 
     * for which <code>i.get().equals(value) == true</code>.
     * Returns the past-the-end of the resulting range.
     * <code>remove</code> is stable, that is, the relative order of the 
     * elements that are not removed is the same as the 
     * relative order in the original range.
     * @param first The beginning of the range
     * @param last The end of the range.
     * @param value Only elements equal to value is removed.
     * @return An iterator pointing past-the-end of the resulting range.
     */
    public static ForwardIterator remove(ForwardIterator first, ForwardIterator last, Object value) {
	first = (ForwardIterator)Algo.find(first, last, value);
	ForwardIterator next = Iter.copy(first);
	next.next();
	return first.cmp(last) ? first : (ForwardIterator)Algo.remove_copy(next, last, first, value);
    }

    /**
     * Remove all elements in the range which satisfy a predicate.
     * <code>remove</code> eliminates all the elements referred to by 
     * iterator <code>i</code> in the range <code>[first,last)</code> 
     * for which <code>pred(i.get()) == true</code>. 
     * Returns the past-the-end of the resulting range.
     * remove is stable, that is, the relative order of the 
     * elements that are not removed is the same as the 
     * relative order in the original range.
     * @param first The beginning of the range
     * @param last The end of the range.
     * @param pred The predicate that must be true for 
     *             elements to be removed
     * @return An iterator pointing past-the-end of the resulting range.
     */
    public static ForwardIterator remove(ForwardIterator first, ForwardIterator last, Predicate1 pred) {
	first = (ForwardIterator)Algo.find(first, last, pred);
	ForwardIterator next = Iter.copy(first);
	next.next();
	return first.cmp(last) ? first : (ForwardIterator)Algo.remove_copy(next, last, first, pred);
    }

    /**
     * Copy all elements in the range except those matching a value.
     * <code>remove_copy</code> copies all the elements referred to by the 
     * iterator <code>i</code> in the range <code>[first,last)</code> 
     * for which <code>i.get().equals(value) == false</code>. 
     * <code>remove_copy</code> returns the past-the-end iterator for the 
     * resulting range.
     * <code>remove_copy</code> is stable, that is, the relative order of the 
     * elements in the resulting range is the same as the 
     * relative order in the original range.
     * @param first The beginning of the range
     * @param last The end of the range.
     * @param value Only elements not equal to value is copied.
     * @return An iterator pointing past-the-end of the resulting range.
     */
    public static OutputIterator remove_copy(InputIterator first, InputIterator last, OutputIterator result, Object value) {
	first = Iter.copy(first);
	result = Iter.copy(result);
	while (!first.cmp(last)) {
	    if (!value.equals(first.get())) {
		result.put(first.get());
		result.next();
	    }
	    first.next();
	}
	return result;
    }

    /**
     * Copy all elements in the range except those which satidfy a predicate.
     * <code>remove_copy</code> copies all the elements referred to by the 
     * iterator <code>i</code> in the range <code>[first,last)</code> 
     * for which <code>pred(i.get()) == false</code>. 
     * <code>remove_copy</code> returns the past-the-end iterator for the 
     * resulting range.
     * <code>remove_copy</code> is stable, that is, the relative order of the 
     * elements in the resulting range is the same as the 
     * relative order in the original range.
     * @param first The beginning of the range
     * @param last The end of the range.
     * @param pred The predicate that must be false for 
     *             elements to be copied.
     * @return An iterator pointing past-the-end of the resulting range.
     */
    public static OutputIterator remove_copy(InputIterator first, InputIterator last, OutputIterator result, Predicate1 pred) {
	first = Iter.copy(first);
	result = Iter.copy(result);
	while (!first.cmp(last)) {
	    if (!pred.compare(first.get())) {
		result.put(first.get());
		result.next();
	    }
	    first.next();
	}
	return result;
    }

    /**
     * Remove all but the first element from every consecutive  
     * group of equal elements.
     * <code>unique</code> eliminates all but the first element 
     * from every consecutive group of equal elements 
     * referred to by the iterator <code>i</code> in 
     * the range <code>[first,last)</code> for which 
     * <code>i.get().equals(i.prev().get()) == true</code>.
     * <code>unique</code> return the end of the resulting range.
     * @param first The beginning of the range
     * @param last The end of the range.
     * @return An iterator pointing past-the-end of the resulting range.
     */
    public static ForwardIterator unique(ForwardIterator first, ForwardIterator last) {
	first = adjacent_find(first, last);
	return (ForwardIterator)unique_copy(first, last, first);
    }

    /**
     * Remove all but the first element from every consecutive  
     * group of equal elements when comparing with a predicate.
     * <code>unique</code> eliminates all but the first element 
     * from every consecutive group of equal elements 
     * referred to by the iterator <code>i</code> in 
     * the range <code>[first,last)</code> for which 
     * <code>pred(i.get(), i.prev().get()) == true</code>.
     * <code>unique</code> return the end of the resulting range.
     * @param first The beginning of the range
     * @param last The end of the range.
     * @param pred The predicate used for comparing elements.
     * @return An iterator pointing past-the-end of the resulting range.
     */
    public static ForwardIterator unique(ForwardIterator first, ForwardIterator last, Predicate2 pred) {
	first = adjacent_find(first, last, pred);
	return (ForwardIterator)unique_copy(first, last, first, pred);
    }

    /**
     * Copy only the first element from every consecutive  
     * group of equal elements.
     * <code>unique_copy</code> copies only the first element 
     * from every consecutive group of equal elements  
     * referred to by the iterator <code>i</code> in 
     * the range <code>[first,last)</code> for which 
     * <code>i.get().equals(i.prev().get()) == true</code>.
     * <code>unique_copy</code> returns the end of the resulting range.
     * @param first The beginning of the range
     * @param last The end of the range.
     * @return An iterator pointing past-the-end of the resulting range.
     */
    public static OutputIterator unique_copy(InputIterator first, InputIterator last, OutputIterator result) {
	first = Iter.copy(first);
	result = Iter.copy(result);
	if (first.cmp(last)) 
	    return result;
	Object value = first.get();
	result.put(value);
	while (!first.next().cmp(last)) {
	    if (!value.equals(first.get())) {
		value = first.get();
		result.next();
		result.put(value);
	    }
	}
	result.next();
	return result;
    }

    /**
     * Copy only the first element from every consecutive  
     * group of equal elements when comparing with a predicate.
     * <code>unique_copy</code> copies only the first element 
     * from every consecutive group of equal elements  
     * referred to by the iterator <code>i</code> in 
     * the range <code>[first,last)</code> for which 
     * <code>pred(i.get(), i.prev().get()) == true</code>.
     * <code>unique_copy</code> returns the end of the resulting range.
     * @param first The beginning of the range
     * @param last The end of the range.
     * @param pred The predicate used for comparing elements.
     * @return An iterator pointing past-the-end of the resulting range.
     */
    public static OutputIterator unique_copy(InputIterator first, InputIterator last, OutputIterator result, Predicate2 pred) {
	first = Iter.copy(first);
	result = Iter.copy(result);
	if (first.cmp(last)) 
	    return result;
	Object value = first.get();
	result.put(value);
	while (!first.next().cmp(last)) {
	    if (!pred.compare(value, first.get())) {
		value = first.get();
		result.next();
		result.put(value);
	    }
	}
	result.next();
	return result;
    }

    /**
     * Reverse the elements in the range.
     * For each integer <code>i <= N/2</code>, <code>reverse</code> 
     * applies swap to all pairs of iterators
     * <code>first+i, last-i-1</code>.
     * @param first The beginning of the range
     * @param last The end of the range.
     */
    public static void reverse(BidirectionalIterator first, BidirectionalIterator last) {
	first = Iter.copy(first);
	last = Iter.copy(last);
	while (true) {
	    if (first.cmp(last))
		return;
	    else {
		last.prev();
		if (first.cmp(last))
		    return;
		else {
		    iter_swap(first, last);
		    first.next();
		}
	    }
	}
    }

    /**
     * Reverse the elements in the range.
     * For each integer <code>i <= N/2</code>, <code>reverse</code>
     * applies swap to all pairs of iterators
     * <code>first+i, last-i-1</code>.
     * @param first The beginning of the range
     * @param last The end of the range.
     */
    public static void reverse(RandomIterator first, RandomIterator last) {
	first = Iter.copy(first);
	last = Iter.copy(last);
	while (first.less(last)) {
	    last.prev();
	    Algo.iter_swap(first, last);
	    first.next();
	}
    }

    /**
     * Shift elements in the range leftwards.
     * For each non-negative integer <code>i < N</code>, <code>rotate</code>
     * places the element from the position <code>first+i</code> into 
     * position <code>first+(i+(last-middle)) % N</code>.
     * <code>[first,middle)</code> and <code>[middle,last)</code> must 
     * be valid ranges.
     * @param first The beginning of the range
     * @param middle The middle of the range.
     * @param last The end of the range.
     */
    public static void rotate(ForwardIterator first, ForwardIterator middle, ForwardIterator last) {
	first = Iter.copy(first);
	middle = Iter.copy(middle);
	if (first.cmp(middle) || middle.cmp(last))
	    return;
	for (ForwardIterator i = Iter.copy(middle); ; ) {	
	    iter_swap(first, i);
	    first.next();
	    if (first.cmp(middle)) {
		if (i.cmp(last))
		    return;
		middle.setTo(i);
	    } else if (i.cmp(last))
		i.setTo(middle);
	}
    }

    /**
     * Shift elements in the range leftwards.
     * For each non-negative integer <code>i < N</code>, rotate
     * places the element from the position <code>first+i</code> into 
     * position <code>first+(i+(last-middle)) % N</code>.
     * <code>[first,middle)</code> and <code>[middle,last)</code> must 
     * be valid ranges.
     * @param first The beginning of the range
     * @param middle The middle of the range.
     * @param last The end of the range.
     */
    public static void rotate(BidirectionalIterator first, BidirectionalIterator middle, BidirectionalIterator last) {
	if (first.cmp(middle) || middle.cmp(last))
	    return;
	reverse(first, middle);
	reverse(middle, last);
	reverse(first, last);
    }

    /**
     * Shift elements in the range leftwards.
     * For each non-negative integer <code>i < N</code>, rotate
     * places the element from the position <code>first+i</code> into 
     * position <code>first+(i+(last-middle)) % N</code>.
     * <code>[first,middle)</code> and <code>[middle,last)</code> must 
     * be valid ranges.
     * @param first The beginning of the range
     * @param middle The middle of the range.
     * @param last The end of the range.
     */
    public static void rotate(RandomIterator first, RandomIterator middle, RandomIterator last) {
	if (first.cmp(middle) || middle.cmp(last))
	    return;
	int n = __gcd(first.distance(last), first.distance(middle));
        RandomIterator nn = Iter.copy(first);
	while (n-- > 0) {	
	    nn.setTo(first);
	    nn.next(n);
	    Algo.__rotate_cycle(first, last, nn, first.distance(middle));
	}
    }

    private static int __gcd(int m, int n) {
	while (n != 0) {
	    int t = m % n;
	    m = n;
	    n = t;
	}
	return m;
    }

    private static void __rotate_cycle(RandomIterator first, RandomIterator last, RandomIterator initial, int shift) {
	Object value = initial.get();
	RandomIterator ptr1 = Iter.copy(initial);
	RandomIterator ptr2 = Iter.copy(initial);
	ptr2.next(shift);

	while (!ptr2.cmp(initial)) {
	    ptr1.put(ptr2.get());
	    ptr1.setTo(ptr2);
	    if (ptr2.distance(last) > shift)
		ptr2.next(shift);
	    else {
		ptr2.setTo(first);
		ptr2.next(shift - ptr2.distance(last));
	    }
	}
	ptr1.put(value);
    }

    /**
     * Shift elements in the range leftwards, placing the result 
     * into another range.
     * <code>rotate_copy</code> copies the range <code>[first,last)</code> to
     * the range <code>[result,...)</code> such that
     * <code>*(result+(i+(last-middle)) % N = *(first+i)</code>.
     * <code>rotate_copy</code> returns a past-the-end iterator of the result.
     * @param first The beginning of the range
     * @param middle The middle of the range.
     * @param last The end of the range.
     */
    public static OutputIterator rotate_copy(ForwardIterator first, ForwardIterator middle, ForwardIterator last, OutputIterator result) {
	return copy(first, middle, copy(middle, last, result));
    }

    /**
     * Randomly reorders the elements in the range.
     * <code>random_shuffle</code> shuffles the elements in 
     * range <code>[first,last)</code> with uniform distribution.
     * Exactly <code>N - 1</code> swaps are done.
     * @param first The beginning of the range
     * @param last The end of the range.
     */
    public static void random_shuffle(RandomIterator first, RandomIterator last) {
	random_shuffle(first, last, new RandomGenerator());
    }

    /**
     * Randomly reorders the elements in the range using 
     * a random generator.
     * <code>random_shuffle</code> shuffles the elements in 
     * range <code>[first,last)</code> with uniform distribution.
     * The random number generator function object <code>rand</code>
     * must take a positive argument <code>n</code> and return 
     * randomly chosen values between 0 and <code>n-1</code>
     * Exactly <code>N - 1</code> swaps are done.
     * @param first The beginning of the range
     * @param last The end of the range.
     * @param rand The random number generator.
     */
    public static void random_shuffle(RandomIterator first, RandomIterator last, RandomGenerator rd) {
	if (first.cmp(last)) 
	    return;
	RandomIterator i = Iter.copy(first);
	for (i.next(); !i.cmp(last); i.next()) 
	    iter_swap(i, Iter.copy(first).next((int)rd.nextLong(first.distance(i) + 1)));
    }

    /**
     * Place all elements satisfying a predicate before those
     * element than to not satisfy the predicate.
     * <code>partition</code> places all the elements in the 
     * range <code>[first,last)</code> that satisfy <code>pred</code> 
     * before all the elements that do not satisfy it.
     * It returns an iterator such that all 
     * elements <code>[first,i)</code>, <code>pred</code> is 
     * <code>true</code> and for all elements <code>[i,last)</code> 
     * <code>pred</code> is <code>false</code>.
     * The predicate is tested exactly <code>N</code> times.
     * @param first The beginning of the range
     * @param last The end of the range.
     * @param pred The predicate used when testing.
     */
    public static BidirectionalIterator partion(BidirectionalIterator first, BidirectionalIterator last, Predicate1 pred) {
	first = Iter.copy(first);
	last = Iter.copy(last);
	while (true) {
	    while (true)
		if (first.cmp(last))
		    return first;
		else if (pred.compare(first.get()))
		    first.next();
		else 
		    break;
	    last.prev();
	    while (true)
		if (first.cmp(last))
		    return first;
		else if (!pred.compare(last.get()))
		    last.prev();
		else
		    break;
	    Algo.iter_swap(first, last);
	    first.next();
	}
    }


    /**
     * Place all elements satisfying a predicate before those
     * element than to not satisfy the predicate while maintaing
     * the relative order of elements.
     * <code>stable_partition</code> places all the elements in the 
     * range <code>[first,last)</code> that satisfy pred before 
     * all the elements that do not satisfy it.
     * It returns an iterator such that all 
     * elements <code>[first,i)</code>, <code>pred</code> is <code>true</code> 
     * and for all elements <code>[i,last)</code> <code>pred</code> 
     * is <code>false</code>.
     * The relative order of the elements in both 
     * groups is preserved.
     * The predicate is tested exactly <code>N</code> times.
     * @param first The beginning of the range
     * @param last The end of the range.
     * @param pred The predicate used when testing.
     */
    public static BidirectionalIterator stable_partion(BidirectionalIterator first, BidirectionalIterator last, Predicate1 pred) {
	int len = first.distance(last);
	Object[] tmpbuf = Algo._get_temp_buffer(len);
	if (tmpbuf == null)
	    return (BidirectionalIterator)_inplace_stable_partition(first, last, pred, len);
	else
	    return (BidirectionalIterator)_stable_partion(first, last, pred, len, tmpbuf);
    }

    private static ForwardIterator _stable_partion(ForwardIterator first, ForwardIterator last, Predicate1 pred, int len, Object[] tmpbuf) {
	first = Iter.copy(first);
	int fill_pointer = 0;

	if (len <= tmpbuf.length) {
	    len = 0;
	    ForwardIterator result1 = Iter.copy(first);
	    int result2 = 0;
	    while (!first.cmp(last) && len < fill_pointer)
		if (pred.compare(first.get())) {
		    result1.put(first.get());
		    result1.next();
		    first.next();
		} else {
		    tmpbuf[result2] = first.get();
		    ++len;
		    ++result2;
		    first.next();
		}
	    if (!first.cmp(last)) {
		int result3 = result2;
		while (!first.cmp(last))
		    if (pred.compare(first.get())) {
			result1.put(first.get());
			result1.next();
			first.next();
		    } else {
			tmpbuf[result3] = first.get();
			++len;
			++result3;
			first.next();
		    }
		fill_pointer = len;
	    }
	    Algo.copy(Array.begin(tmpbuf), Array.begin(tmpbuf, len), result1);
	    return result1;
	}
	ForwardIterator middle = Iter.copy(first);
	Algo.advance(middle, len / 2);
	ForwardIterator first_cut = 
		_stable_partion(first, middle, pred, len / 2, tmpbuf);
	ForwardIterator second_cut = 
		_stable_partion(middle, last, pred, len - len / 2, tmpbuf);
	Algo.rotate(first_cut, middle, second_cut);	
	len = middle.distance(second_cut);
	Algo.advance(first_cut, len);
	return first_cut;
    }

    private static ForwardIterator _inplace_stable_partition(ForwardIterator first, ForwardIterator last, Predicate1 pred, int len) {
	if (len == 1)
	    return pred.compare(first.get()) ? last : first;
	ForwardIterator middle = Iter.copy(first);
	Algo.advance(middle, len/2);
	ForwardIterator first_cut = 
		_inplace_stable_partition(first, middle, pred, len / 2);
	ForwardIterator second_cut = 
		_inplace_stable_partition(middle, last, pred, len - len / 2);
	Algo.rotate(first_cut, middle, second_cut);	
	len = middle.distance(second_cut);
	Algo.advance(first_cut, len);
	return first_cut;
    }


    /**
     * Sort the elements in the range according to the 
     * comparison predicate pred.
     * <code>sort</code> sorts the element in the range 
     * <code>[first,last)</code>. 
     * It does approximatly <code>NlogN</code> comparisons on the average. 
     * If the worst case behaivior is important <code>stable_sort</code>
     * or <code>partial_sort</code> should be used.
     * @param first The beginning of the range
     * @param last The end of the range.
     * @param pred The predicate used when testing.
     */
    public static void sort(RandomIterator first, RandomIterator last, Predicate2 pred) {
	_quick_sort_loop(first, last, pred);
	_final_insertion_sort(first, last, pred);
    }

    private static void _quick_sort_loop(RandomIterator first, RandomIterator last, Predicate2 pred) {
	while (first.distance(last) > sortThreshold) {
	    RandomIterator cut = _unguarded_partition(first, last, 
		_median(first.get(), first.get(first.distance(last) / 2), 
		last.get(-1), pred), pred);
	    if (first.distance(cut) >= cut.distance(last)) {
		_quick_sort_loop(cut, last, pred);
		last = cut;
	    } else {
		_quick_sort_loop(first, cut, pred);
		first = cut;
	    }
	}
    }

    private static RandomIterator _unguarded_partition(RandomIterator first, RandomIterator last, Object pivot, Predicate2 pred) {
	first = Iter.copy(first);
	last = Iter.copy(last);

	while (true) {
	    while (pred.compare(first.get(), pivot))
		first.next();
	    last.prev();
	    while (pred.compare(pivot, last.get()))
		last.prev();
	    if (!first.less(last))
		return first;
	    iter_swap(first, last);
	    first.next();
	}
    }

    private static Object _median(Object a, Object b, Object c, Predicate2 pred) {
	if (pred.compare(a, b))
	    if (pred.compare(b, c))
		return b;
	    else if (pred.compare(a, c))
		return c;
	    else
		return a;
	else if (pred.compare(a, c))
	    return a;
	else if (pred.compare(b, c))
	    return c;
	else
	    return b;
    }

    private static void _insertion_sort(RandomIterator first, RandomIterator last, Predicate2 pred) {
	if (first.cmp(last))
	    return;
	RandomIterator i = Iter.copy(first);
	for (i.next(); !i.cmp(last); i.next())
	    _linear_insert(first, i, pred);
    }

    private static void _linear_insert(RandomIterator first, RandomIterator last, Predicate2 pred) {
	Object value = last.get();
	if (pred.compare(value, first.get())) {
	    copy_backward(first, last, Iter.copy(last).next(1));
	    first.put(value);
	} else
	    _unguarded_linear_insert(last, value, pred);
    }

    private static void _unguarded_linear_insert(RandomIterator last, Object value, Predicate2 pred) {
	last = Iter.copy(last);
	RandomIterator next = Iter.copy(last);
	next.prev();
	while (pred.compare(value, next.get())) {
	    last.put(next.get());
	    last.prev();
	    next.prev();
	}
	last.put(value);
    }
	    

    private static void _final_insertion_sort(RandomIterator first, RandomIterator last, Predicate2 pred) {
	if (first.distance(last) > sortThreshold) {
	    _insertion_sort(first, Iter.copy(first).next(sortThreshold), pred);
	    _unguarded_insertion_sort(Iter.copy(first).next(sortThreshold), last, pred);
	} else
	    _insertion_sort(first, last, pred);
    }

    private static void _unguarded_insertion_sort(RandomIterator first, RandomIterator last, Predicate2 pred) {
	first = Iter.copy(first);
	while (!first.cmp(last)) {
	    _unguarded_linear_insert(first, first.get(), pred);
	    first.next();
	}
    }


    /**
     * Sort the elements in the range according to the 
     * comparison predicate pred.
     * <code>stable_sort</code> sorts the element in the range 
     * <code>[first,last)</code>. 
     * It is stable, that is, the relative order of the equal 
     * elements is preserved. It does at most <code>N(logN)<sup>2</sup></code>
     * comparisons; if enough extra memory is available, it is 
     * <code>NlogN</code>.
     * @param first The beginning of the range
     * @param last The end of the range.
     * @param pred The predicate used when testing.
     */
    public static void stable_sort(RandomIterator first, RandomIterator last, Predicate2 pred) {
	Object[] tmpbuf = _get_temp_buffer(first.distance(last));
	if (tmpbuf == null)
	    _inplace_stable_sort(first, last, pred);
	else
	    _stable_sort(first, last, tmpbuf, pred);
    }


    private static void _stable_sort(RandomIterator first, RandomIterator last, Object[] tmpbuf, Predicate2 pred) {
	int len = min(tmpbuf.length, first.distance(last));
	copy(first, Iter.copy(first).next(len), Array.begin(tmpbuf));
	_stable_sort_adaptive(first, last, tmpbuf, pred);
    }

    private static void _stable_sort_adaptive(RandomIterator first, RandomIterator last, Object[] tmpbuf, Predicate2 pred) {
	int len = (first.distance(last) + 1) / 2;
	RandomIterator middle = Iter.copy(first);
	middle.next(len);
	if (len > tmpbuf.length) {
	    _stable_sort_adaptive(first, middle, tmpbuf, pred);
	    _stable_sort_adaptive(middle, last, tmpbuf, pred);
	} else {
	    _merge_sort_with_buffer(first, middle, tmpbuf, pred);
	    _merge_sort_with_buffer(middle, last, tmpbuf, pred);
	}
	_merge_adaptive(first, middle, last, first.distance(middle), middle.distance(last), Array.begin(tmpbuf), tmpbuf.length, pred);
    }

    private static void _merge_sort_with_buffer(RandomIterator first, RandomIterator last, Object[] tmpbuf, Predicate2 pred) {
	int len = first.distance(last);
	int step_size = sortChunkSize;
	_chunk_insertion_sort(first, last, step_size, pred);
	while (step_size < len) {
	    _merge_sort_loop(first, last, Array.begin(tmpbuf), step_size, pred);
	    step_size *= 2;
	    _merge_sort_loop(Array.begin(tmpbuf), Array.end(tmpbuf), first, step_size, pred);
	    step_size *= 2;
	}
    }

    private static void _chunk_insertion_sort(RandomIterator first, RandomIterator last, int chunk_size, Predicate2 pred) {
	while (first.distance(last) >= chunk_size) {
	    _insertion_sort(first, Iter.copy(first).next(chunk_size), pred);
	    first.next(chunk_size);
	}
	_insertion_sort(first, last, pred);
    }

    private static void _merge_sort_loop(RandomIterator first, RandomIterator last, RandomIterator result, int step_size, Predicate2 pred) {
	int two_step = 2 * step_size;
	first = Iter.copy(first);
	while (first.distance(last) >= two_step) {
	    result = (RandomIterator)merge(first, Iter.copy(first).next(step_size), 
		Iter.copy(first).next(step_size), 
		Iter.copy(first).next(two_step), result, pred);
	    first.next(two_step);
	}
	step_size = min(first.distance(last), step_size);
	merge(first, Iter.copy(first).next(step_size), 
			Iter.copy(first).next(step_size), last, result, pred);
    }

    private static void _inplace_stable_sort(RandomIterator first, RandomIterator last, Predicate2 pred) {
	if (first.distance(last) < 15) {
	    _insertion_sort(first, last, pred);
	    return;
	}
	RandomIterator middle = Iter.copy(first).next(first.distance(last) / 2);
	_inplace_stable_sort(first, middle, pred);
	_inplace_stable_sort(middle, last, pred);
	_merge_without_buffer(first, middle, last, first.distance(middle),
		middle.distance(last), pred);
    }


    /*
     * Sort the elements in the range according to the 
     * comparison predicate pred.
     * <code>partial_sort</code> places the first <code>middle-first</code>
     * sorted elements from the range <code>[first,last)</code> into
     * the range <code>[first,middle)</code>. 
     * The rest of the elements in the range <code>[middle,last)</code>
     * is placed in an undefined order.
     * It does appriximately <code>N*log(middle-first)</code>
     * comparisons.
     * @param first The beginning of the range
     * @param middle The middle of the range.
     * @param last The end of the range.
     * @param pred The predicate used when testing.
     */
    private static void partial_sort(RandomIterator first, RandomIterator middle, RandomIterator last, Predicate2 pred) {
	make_heap(first, middle, pred);	
	for (RandomIterator i = Iter.copy(middle); i.less(last); i.next()) {
	    if (pred.compare(i.get(), first.get()))
		_pop_heap(first, middle, i, i.get(), pred);
	}
	sort_heap(first, middle, pred);
    }

    /*
     * Sort the elements in the range according to the 
     * comparison predicate pred.
     * <code>partial_sort_copy</code> places the first 
     * <code>NN = min(last-first,result_last-result_first)</code>
     * sorted elements into the range <code>[result_first,result_first+NN)</code>.
     * It returns either result_last ot result_first+NN whichever is smaller. 
     * It takes appriximatly NlogNN comparisons.
     * @param first The beginning of the range
     * @param middle The middle of the range.
     * @param result_first The beginning of the result range
     * @param result_last The middle of the result range.
     * @param pred The predicate used when testing.
     */
    private static RandomIterator partial_sort_copy(RandomIterator first, RandomIterator last, RandomIterator result_first, RandomIterator result_last, Predicate2 pred) {
	if (result_first.cmp(result_last))
	    return result_last;
	RandomIterator result_real_last = Iter.copy(result_first);
	first = Iter.copy(first);
	while (!first.cmp(last) && !result_real_last.cmp(result_last)) {
	    result_real_last.put(first.get());
	    result_real_last.next();
	    first.next();
	}
	while (!first.cmp(last)) {
	    if (pred.compare(first.get(), result_first.get())) {
		_adjust_heap(result_first, 0, 
			result_real_last.distance(result_first),
			first.get(), pred);
	    }
	    first.next();
	}
	sort_heap(result_first, result_real_last, pred);
	return result_real_last;
    }

    /**
     * Place an element of a sequence in the location where it 
     * would be if the sequence was sorted.
     * After <code>nth_element</code> the element in the position
     * pointed to by <code>nth</code> is the element that would be in that  
     * position if the whole range was sorted. Also for any iterator
     * <code>i</code> in the range <code>[first, nth)</code> and 
     * any iterator <code>j</code> in the range <code>[nth, last)</code> 
     * it holds that <code>pred(i.get(), j.get()) == false</code>.
     * It is linear on the average.
     * @param first The beginning of the range.
     * @param nth The position into the range.
     * @param last The end of the range.
     * @param pred The predicate used when testing.
     */
    public static void nth_element(RandomIterator first, RandomIterator nth, RandomIterator last, Predicate2 pred) {
	first = Iter.copy(first);
	last = Iter.copy(last);

	while (first.distance(last) > 3) {
	    RandomIterator cut = _unguarded_partition(first, last, 
		_median(first.get(), first.get(first.distance(last)/2),
					last.get(-1), pred), pred);
	    if (!nth.less(cut)) 
		first.setTo(cut);
	    else
		last.setTo(cut);
	}
	_insertion_sort(first, last, pred);
    }

    /**
     * Find the first position into which a value can be inserted 
     * without violation the ordering.
     * <code>lower_bound</code> finds the first position where 
     * <code>value</code> can be inserted. <code>lower_bound</code> returns
     * the furthermost iterator <code>i</code> in the range 
     * <code>[first,last)</code> such that for any iterator <code>j</code>
     * in the range <code>[first,i)</code> the following condition holds: 
     * <code>pred(j.get(), value) == true</code>.
     * At most <code>log(N) + 1</code> comparisons are done.
     * @param first The beginning of the range.
     * @param last The end of the range.
     * @param value The value tested against.
     * @param pred The predicate used when testing.
     */
    public static ForwardIterator lower_bound(ForwardIterator first, ForwardIterator last, Object value, Predicate2 pred) {
	first = Iter.copy(first);
	int len = first.distance(last);
	int half;
	ForwardIterator middle = Iter.copy(first);

	while (len > 0) {
	    half = len / 2;
	    middle.setTo(first);
	    advance(middle, half);
	    if (pred.compare(middle.get(), value)) {
		first.setTo(middle);
	    	first.next();
		len = len - half - 1;
	    } else
		len = half;
	}
	return first;
    }

    /**
     * Find the furthermost position into which a value can be inserted 
     * without violation the ordering.
     * <code>upper_bound</code> finds the furthermost position where 
     * <code>value</code> can be inserted. <code>upper_bound</code> returns
     * the furthermost iterator <code>i</code> in the range 
     * <code>[first,last)</code> such that for any iterator <code>j</code>
     * in the range <code>[first,i)</code> the following condition holds: 
     * <code>pred(value, j.get()) == false</code>.
     * At most <code>log(N) + 1</code> comparisons are done.
     * @param first The beginning of the range.
     * @param last The end of the range.
     * @param value The value tested against.
     * @param pred The predicate used when testing.
     */
    public static ForwardIterator upper_bound(ForwardIterator first, ForwardIterator last, Object value, Predicate2 pred) {
	first = Iter.copy(first);
	int len = first.distance(last);
	int half;
	ForwardIterator middle = Iter.copy(first);

	while (len > 0) {
	    half = len / 2;
	    middle.setTo(first);
	    advance(middle, half);
	    if (pred.compare(value, middle.get()))
		len = half;
	    else {
		first.setTo(middle);
	    	first.next();
		len = len - half - 1;
	    } 

	}
	return first;
    }

    /**
     * Find the largest subrange into which a value can be inserted 
     * without violation the ordering.
     * <code>equal_range</code> finds the largest subrange 
     * <code>[i,j)</code> such that the value <code>value</code> can be 
     * at any iterator <code>k</code> in it.
     * <code>k</code> satisfy the condition
     * <code>pred(k.get(), value) == false && pred(value, k.get())</code>.
     * At most <code>2 * log(N) + 1</code> comparisons are done.
     * @param first The beginning of the range.
     * @param last The end of the range.
     * @param value The value tested against.
     * @param pred The predicate used when testing.
     */
    public static Pair equal_range(ForwardIterator first, ForwardIterator last, Object value, Predicate2 pred) {
	int len = first.distance(last);
	int half;
	first = Iter.copy(first);
	ForwardIterator middle = Iter.copy(first);
	ForwardIterator left = Iter.copy(first); 
	ForwardIterator right = Iter.copy(first);
	
	while (len > 0) {
	    half = len / 2;
	    middle.setTo(first);
	    advance(middle, half);
	    if (pred.compare(middle.get(), value)) {
		first.setTo(middle);
		first.next();
		len = len - half - 1;	
	    } else if (pred.compare(value, middle.get()))
		len = half;
	    else {
		left = lower_bound(first, middle, value, pred);
		advance(first, len);
		middle.next();
		right = upper_bound(middle, first, value, pred);
		return new Pair(left, right);
	    }
	}
	return new Pair(first, first);
    }

    /**
     * Returns true is the value is in the range.
     * <code>binary_search</code> returns <code>true</code> if there 
     * is an iterator <code>i</code> in the range <code>[first,last)</code>
     * that satisfy the condition 
     * <code>pred(i.get(), value) == false && pred(value, i.get()) == false</code>.
     * At most <code>log(N) + 1</code> comparisons are done.
     * @param first The beginning of the range.
     * @param last The end of the range.
     * @param value The value tested against.
     * @param pred The predicate used when testing.
     */
    public static boolean binary_search(ForwardIterator first, ForwardIterator last, Object value, Predicate2 pred) {
	ForwardIterator i = lower_bound(first, last, value, pred);
	return !i.cmp(last) && !pred.compare(value, i.get());
    }


    /**
     * Merge the two sorted ranges and place the result into the result range.
     * <code>merge</code> merges the two sorted ranges 
     * <code>[first1,last1)</code> and <code>[first2,last2)</code> into
     * the range <code>[result,...)</code>. The merge is <i>stable</i>, 
     * that is, for equal elements in the two ranges, the element from
     * the first range always precede the element from the second range.
     * <code>merge</code> return past-the-end of the result range.
     * The result of <code>merge</code> is undefined if the resulting
     * range overlaps with either of the orignal ranges.
     * @param first1 The beginning of the first range.
     * @param last1 The end of the first range.
     * @param first2 The beginning of the second range.
     * @param last2 The end of the second range.
     * @param result The beginning of the resulting range.
     * @param pred The predicate used when testing.
     * @return The end of the resulting range.
     */
    public static OutputIterator merge(InputIterator first1, InputIterator last1, InputIterator first2, InputIterator last2, OutputIterator result, Predicate2 pred) {
	first1 = Iter.copy(first1);
	first2 = Iter.copy(first2);
	result = Iter.copy(result);
	while (!first1.cmp(last1) && ! first2.cmp(last2)) {
	    if (pred.compare(first2.get(), first1.get())) {
		result.put(first2.get());
		result.next();
		first2.next();
	    } else {
		result.put(first1.get());
		result.next();
		first1.next();
	    }
	}
	return copy(first2, last2, copy(first1, last1, result));
    }
    private static void _merge_without_buffer(BidirectionalIterator first, BidirectionalIterator middle, BidirectionalIterator last, int len1, int len2, Predicate2 pred) {
	if (len1 == 0 || len2 == 0)
	    return;
	if (len1 + len2 == 2) {
	    if (pred.compare(middle.get(), first.get()))
		iter_swap(first, middle);
	    return;
	}
	BidirectionalIterator first_cut = Iter.copy(first);
	BidirectionalIterator second_cut = Iter.copy(middle);
	int len11 = 0;
	int len22 = 0;
	if (len1 > len2) {
	    len11 = len1 / 2;
	    advance(first_cut, len11);
	    second_cut = (BidirectionalIterator)lower_bound(middle, last, first_cut.get(), pred);
	    len22 += middle.distance(second_cut);
	} else {
	    len22 = len2 / 2;
	    advance(second_cut, len22);
	    first_cut = (BidirectionalIterator)upper_bound(first, middle, second_cut.get(), pred);
	    len11 += first.distance(first_cut);	
	}
	rotate(first_cut, middle, second_cut);
	BidirectionalIterator new_middle = Iter.copy(first_cut);
	advance(new_middle, len22);
	_merge_without_buffer(first, first_cut, new_middle, len11, len22, pred);
	_merge_without_buffer(new_middle, second_cut, last, len1 - len11, 
		len2 - len22, pred);	
    }

    private static BidirectionalIterator _rotate_adaptive(BidirectionalIterator first, BidirectionalIterator middle, BidirectionalIterator last, int len1, int len2, BidirectionalIterator buffer, int buffer_size) {
	first = Iter.copy(first);

	BidirectionalIterator buffer_end;
	if (len1 > len2 && len2 <= buffer_size) {
	    buffer_end = (BidirectionalIterator)copy(middle, last, buffer);
	    copy_backward(first, middle, last);
	    return (BidirectionalIterator)copy(buffer, buffer_end, last);
	} else if (len1 <= buffer_size) {
	    buffer_end = (BidirectionalIterator)copy(first, middle, buffer);
	    copy(middle, last, first);
	    return (BidirectionalIterator)copy_backward(buffer, buffer_end, last);
	} else {
	    rotate(first, middle, last);
	    advance(first, len2);
	    return first;
	}
    }

    private static BidirectionalIterator _merge_backward(BidirectionalIterator first1, BidirectionalIterator last1, BidirectionalIterator first2, BidirectionalIterator last2, BidirectionalIterator result, Predicate2 pred) {
	last1 = Iter.copy(last1);
	last2 = Iter.copy(last2);
	result = Iter.copy(result);
	if (first1.cmp(last1))
	    return copy_backward(first2, last2, result);
	if (first2.cmp(last2))
	    return copy_backward(first1, last1, result);
	last1.prev();
	last2.prev();
	while (true) {
	    if (pred.compare(last2.get(), last1.get())) {
		result.prev();
		result.put(last1.get());
		if (first1.cmp(last1)) {
		    last2.next();
		    return copy_backward(first2, last2, result);
		}
		last1.prev();
	    } else {
		result.prev();
		result.put(last2.get());
		if (first2.cmp(last2)) {
		    last1.next();
		    return copy_backward(first1, last1, result);
		}
		last2.prev();
	    }
	}
    }

    private static void _merge_adaptive(BidirectionalIterator first, BidirectionalIterator middle, BidirectionalIterator last, int len1, int len2, BidirectionalIterator buffer, int buffer_size, Predicate2 pred) {
	if (len1 <= len2 && len1 <= buffer_size) {
	    BidirectionalIterator end_buffer = 
			(BidirectionalIterator)copy(first, middle, buffer);
	    merge(buffer, end_buffer, middle, last, first, pred);
	} else if (len2 <= buffer_size) {
	    BidirectionalIterator end_buffer = 
			(BidirectionalIterator)copy(middle, last, buffer);
	    _merge_backward(first, middle, buffer, end_buffer, last, pred);
	} else {
	    BidirectionalIterator first_cut = Iter.copy(first);
	    BidirectionalIterator second_cut = Iter.copy(middle);
	    int len11 = 0;
	    int len22 = 0;
	    if (len1 > len2) {
		len11 = len1 / 2;

		len11 = len1 / 2;
	        advance(first_cut, len11);
	        second_cut = (BidirectionalIterator)lower_bound(middle, last, 
			first_cut.get(), pred);
	        len22 += middle.distance(second_cut);	
	    } else {
	        len22 = len2 / 2;
	        advance(second_cut, len22);
	        first_cut = (BidirectionalIterator)upper_bound(first, middle, 
			second_cut.get(), pred);
	        len11 += first.distance(first_cut);	
	    }
	    BidirectionalIterator new_middle = _rotate_adaptive(first_cut, middle, 
			second_cut, len1 - len11, len22, buffer, buffer_size);
	    _merge_adaptive(first, first_cut, new_middle, len11, len22, 
			buffer, buffer_size, pred);
	    _merge_adaptive(new_middle, second_cut, last, len1 - len11, 
			len2 - len22, buffer, buffer_size, pred);	
	}
    }

    /**
     * Merge the two sorted ranges and place the result into the result range.
     * <code>inplace_merge</code> merges the two sorted ranges 
     * <code>[first1,last1)</code> and <code>[first2,last2)</code> into
     * the range <code>[result,...)</code>. The merge is <i>stable</i>, 
     * that is, for equal elements in the two ranges, the element from
     * the first range always precede the element from the second range.
     * <code>merge</code> return past-the-end of the result range.
     * The result of <code>merge</code> is undefined if the resulting
     * range overlaps with either of the orignal ranges.
     * @param first1 The beginning of the first range.
     * @param last1 The end of the first range.
     * @param first2 The beginning of the second range.
     * @param last2 The end of the second range.
     * @param result The beginning of the resulting range.
     * @param pred The predicate used when testing.
     * @return The end of the resulting range.
     */
    public static void inplace_merge(BidirectionalIterator first, BidirectionalIterator middle, BidirectionalIterator last, Predicate2 pred) {
	if (first.cmp(middle) || middle.cmp(last))
	    return;
	int len1 = first.distance(middle);
	int len2 = middle.distance(last);
	Object[] tmpbuf = _get_temp_buffer(len1 + len2);
	if (tmpbuf == null)
	    _merge_without_buffer(first, middle, last, len1, len2, pred);
	else
	   _inplace_merge(first, middle, last, len1, len2, tmpbuf, pred);
    }

    private static void _inplace_merge(BidirectionalIterator first, BidirectionalIterator middle, BidirectionalIterator last, int len1, int len2, Object[] p, Predicate2 pred) {
	int len = min(p.length, len1 + len2);
	fill(Array.begin(p), len, first.get());
	_merge_adaptive(first, middle, last, len1, len2, 
			Array.begin(p), p.length, pred);
    }

	    
    /**
     * Return <code>true</code> if all the elements in range2 also is in range2.
     * <code>includes</code> return <code>true</code> if every element in the 
     * range <code>[first2,last2)</code> is contained in the range
     * <code>[first1,last1)</code>. 
     * It returns <code>false</code> otherwise.
     * At most <code>(N1+N2)*2-1</code> comparisons are performed.
     * @param first1 The beginning of the first range.
     * @param last1 The end of the first range.
     * @param first2 The beginning of the second range.
     * @param last2 The end of the second range.
     * @param pred The predicate used when testing.
     * @return true if range2 in included in range1.
     */
    public static boolean includes(InputIterator first1, InputIterator last1, InputIterator first2, InputIterator last2, Predicate2 pred) {
	first1 = Iter.copy(first1);
	first2 = Iter.copy(first2);
	while (!first1.cmp(last1) && !first2.cmp(last2)) {
	    if (pred.compare(first2.get(), first1.get()))
		return false;
	    else if (pred.compare(first1.get(), first2.get()))
		first1.next();
	    else {
		first1.next();
		first2.next();
	    }
	}
	return first2.cmp(last2);
    }

    /**
     * Create the union of the two ranges.
     * <code>set_union</code> constructs a sorted union of the elements
     * from the two ranges. It returns the end of the constructed range.
     * <code>set_union</code> is stable, that is, if an element is present
     * in both ranges, the one from the first range is copied.
     * At most <code>(N1+N2)*2-1</code> comparisons are performed.
     * The result of <code>set_union</code> is undefined if the resulting 
     * range overlaps with either of original ranges.
     * @param first1 The beginning of the first range.
     * @param last1 The end of the first range.
     * @param first2 The beginning of the second range.
     * @param last2 The end of the second range.
     * @param result The beginning of the resulting range.
     * @param pred The predicate used when testing.
     * @return The end of the result range.
     */
    public static OutputIterator set_union(InputIterator first1, InputIterator last1, InputIterator first2, InputIterator last2, OutputIterator result, Predicate2 pred) {
	first1 = Iter.copy(first1);
	first2 = Iter.copy(first2);
	result = Iter.copy(result);
	while (!first1.cmp(last1) && !first2.cmp(last2)) {
	    if (pred.compare(first1.get(), first2.get())) {
		result.put(first1.get());
		result.next();
		first1.next();
	    } else if (pred.compare(first2.get(), first1.get())) {
		result.put(first2.get());
		result.next();
		first2.next();
	    } else {
		result.put(first1.get());
		result.next();
		first1.next();
		first2.next();
	    }
	}
	return copy(first2, last2, copy(first1, last1, result));
    }

    /**
     * Create the intersection of the two ranges.
     * <code>set_intersection</code> constructs a sorted intersection of the elements
     * from the two ranges. It returns the end of the constructed range.
     * <code>set_intersection</code> is guarenteed to be stable, that is, 
     * if an element is present in both ranges, the one from the first range 
     * is copied.
     * At most <code>(N1+N2)*2-1</code> comparisons are performed.
     * The result of <code>set_intersection</code> is undefined if the resulting 
     * range overlaps with either of original ranges.
     * @param first1 The beginning of the first range.
     * @param last1 The end of the first range.
     * @param first2 The beginning of the second range.
     * @param last2 The end of the second range.
     * @param result The beginning of the resulting range.
     * @param pred The predicate used when testing.
     * @return The end of the result range.
     */
    public static OutputIterator set_intersection(InputIterator first1, InputIterator last1, InputIterator first2, InputIterator last2, OutputIterator result, Predicate2 pred) {
	first1 = Iter.copy(first1);
	first2 = Iter.copy(first2);
	result = Iter.copy(result);

	while (!first1.cmp(last1) && !first2.cmp(last2)) {
	    if (pred.compare(first1.get(), first2.get())) 
		first1.next();
	    else if (pred.compare(first2.get(), first1.get()))
		first2.next();
	    else {
		result.put(first1.get());
		result.next();
		first1.next();
		first2.next();
	    }
	}
	return result;
    }

    /**
     * Create the difference of the two ranges.
     * <code>set_difference</code> constructs a sorted difference of the elements
     * from the two ranges. It returns the end of the constructed range.
     * At most <code>(N1+N2)*2-1</code> comparisons are performed.
     * The result of <code>set_difference</code> is undefined if the resulting 
     * range overlaps with either of original ranges.
     * @param first1 The beginning of the first range.
     * @param last1 The end of the first range.
     * @param first2 The beginning of the second range.
     * @param last2 The end of the second range.
     * @param result The beginning of the resulting range.
     * @param pred The predicate used when testing.
     * @return The end of the result range.
     */
    public static OutputIterator set_difference(InputIterator first1, InputIterator last1, InputIterator first2, InputIterator last2, OutputIterator result, Predicate2 pred) {
	first1 = Iter.copy(first1);
	first2 = Iter.copy(first2);
	result = Iter.copy(result);
	while (!first1.cmp(last1) && !first2.cmp(last2)) {
	    if (pred.compare(first1.get(), first2.get())) {
		result.put(first1.get());
		result.next();
		first1.next();
	    } else if (pred.compare(first2.get(), first1.get()))
		first2.next();
	    else {
		first1.next();
		first2.next();
	    }
	}
	return copy(first1, last1, result);
    }

    /**
     * Create the symmetric difference of the two ranges.
     * <code>set_symmetric_difference</code> constructs a sorted symmetric 
     * difference of the elements from the two ranges. 
     * It returns the end of the constructed range.
     * At most <code>(N1+N2)*2-1</code> comparisons are performed.
     * The result of <code>set_symmetric_difference</code> is undefined if the resulting 
     * range overlaps with either of original ranges.
     * @param first1 The beginning of the first range.
     * @param last1 The end of the first range.
     * @param first2 The beginning of the second range.
     * @param last2 The end of the second range.
     * @param result The beginning of the resulting range.
     * @param pred The predicate used when testing.
     * @return The end of the result range.
     */
    public static OutputIterator set_symmetric_difference(InputIterator first1, InputIterator last1, InputIterator first2, InputIterator last2, OutputIterator result, Predicate2 pred) {
	first1 = Iter.copy(first1);
	first2 = Iter.copy(first2);
	result = Iter.copy(result);
	while (!first1.cmp(last1) && !first2.cmp(last2)) {
	    if (pred.compare(first1.get(), first2.get())) {
		result.put(first1.get());
		result.next();
		first1.next();
	    } else if (pred.compare(first2.get(), first1.get())) {
		result.put(first2.get());
		result.next();
		first2.next();
	    } else {
		first1.next();
		first2.next();
	    }
	}
	return copy(first2, last2, copy(first1, last1, result));
    }


    /**
     * Place a new value into the heap.
     * <code>push_heap</code> assumes the range <code>[first,last.prev())</code>
     * is a valid heap and properly places the value in the location
     * <code>last.prev()</code> into the resulting heap 
     * <code>[first,last)</code>. 
     * At most <code>log(N)</code> comparisons are performed.
     * @param first The beginning of the range.
     * @param last The end of the range.
     * @param pred The predicate used when testing.
     */
    public static void push_heap(RandomIterator first, RandomIterator last, Predicate2 pred) {
	_push_heap(first, first.distance(last) - 1, 0, last.get(-1), pred);
    }

    private static void _push_heap(RandomIterator first, int holeIndex, int topIndex, Object value, Predicate2 pred) {
	int parent = (holeIndex - 1) / 2;
	while (holeIndex > topIndex && pred.compare(first.get(parent), value)) {
	    first.put(holeIndex, first.get(parent));
	    holeIndex = parent;
	    parent = (holeIndex - 1) / 2;
	}
	first.put(holeIndex, value);
    }

    /**
     * Remove the largest value from the heap.
     * <code>pop_heap</code> assumes the range <code>[first,last)</code>
     * is a valid heap, then swaps the value in the location
     * <code>first()</code> with the value in the location 
     * <code>last.prev()</code> and makes <code>[first,last.prev())</code>
     * into a heap.
     * At most <code>2*log(N)</code> comparisons are performed.
     * @param first The beginning of the range.
     * @param last The end of the range.
     * @param pred The predicate used when testing.
     */
    public static void pop_heap(RandomIterator first, RandomIterator last, Predicate2 pred) {
	last = Iter.copy(last);
	last.prev();
	_pop_heap(first, last, last, last.get(), pred);
    }

    private static void _pop_heap(RandomIterator first, RandomIterator last, RandomIterator result, Object value, Predicate2 pred) {
	result.put(first.get());
	_adjust_heap(first, 0, first.distance(last), value, pred);
    }


    /**
     * Create a heap out of a sequence.
     * <code>make_heap</code> construct a heap out of the range 
     * <code>[first,last)</code>.
     * At most <code>3*N</code> comparisons are performed.
     * @param first The beginning of the range.
     * @param last The end of the range.
     * @param pred The predicate used when testing.
     */
    public static void make_heap(RandomIterator first, RandomIterator last, Predicate2 pred) {
	if (first.distance(last) < 2)
	    return;
	int len = first.distance(last);
	int parent = (len - 2)/2;
	while (true) {
	    _adjust_heap(first, parent, len, first.get(parent), pred);
	    if (parent == 0)
		return;
	    parent--;
	}
    }

    private static void _adjust_heap(RandomIterator first, int holeIndex, int len, Object value, Predicate2 pred) {
	int topIndex = holeIndex;
	int secondChild = 2 * holeIndex + 2;
	while (secondChild < len) {
	    if (pred.compare(first.get(secondChild), first.get(secondChild - 1)))
		secondChild--;
	    first.put(holeIndex, first.get(secondChild));
	    holeIndex = secondChild;
	    secondChild = 2 * (secondChild + 1);
	}
	if (secondChild == len) {
	    first.put(holeIndex, first.get(secondChild - 1));
	    holeIndex = secondChild - 1;
	}
	_push_heap(first, holeIndex, topIndex, value, pred);
    }

    /**
     * Turns the heap into a sorted sequence.
     * <code>sort_heap</code> sort the elements in the heap
     * <code>[first,last)</code>.
     * At most <code>NlogN</code> comparisons are performed.
     * <code>sort_heap</code> is not stable.
     * @param first The beginning of the range.
     * @param last The end of the range.
     * @param pred The predicate used when testing.
     */
    public static void sort_heap(RandomIterator first, RandomIterator last, Predicate2 pred) {
	last = Iter.copy(last);
	while (first.distance(last) > 1) {
	    pop_heap(first, last, pred);
	    last.prev();
	}
    }


    /**
     * Returns the smaller of the two integers.
     */
    public static int min(int a, int b) {
	return a < b ? a : b;
    }

    /**
     * Returns the larger of the two integers.
     */
    public static int max(int a, int b) {
	return a < b ? b : a;
    }

    /**
     * Returns the smaller of the two characters.
     */
    public static char min(char a, char b) {
	return a < b ? a : b;
    }

    /**
     * Returns the larger of the two characters.
     */
    public static char max(char a, char b) {
	return a < b ? b : a;
    }

    /**
     * Returns the smaller of two objects.
     */
    public static Object min(Object a, Object b, Predicate2 pred) {
	return pred.compare(b, a) ? b : a;
    }

    /**
     * Returns the larger of two objects.
     */
    public static Object max(Object a, Object b, Predicate2 pred) {
	return pred.compare(a, b) ? b : a;
    }


    /**
     * Returns the first iterator pointing to the maximal element in the range.
     * <code>max_element</code> return the first iterator <code>i</code>
     * in the range <code>[first,last)</code> such that for any
     * iterator <code>j</code> in the range <code>[first,last)</code> 
     * the following condition holds: 
     * <code>pred(i.get(), j.get()) == false</code>.
     * Exactly <code>max(N-1, 0)</code> comparisons are performed.
     * @param first The beginning of the range.
     * @param last The end of the range.
     * @param pred The predicate used when testing.
     */
    public static ForwardIterator max_element(ForwardIterator first, ForwardIterator last, Predicate2 pred) {
	first = Iter.copy(first);
	if (first.cmp(last))
	    return first;
	ForwardIterator result = Iter.copy(first);
	while (!first.next().cmp(last))
	    if (pred.compare(result.get(), first.get()))
		result.setTo(first);
	return result;
    }

    /**
     * Returns the first iterator pointing to the minimal element in the range.
     * <code>min_element</code> return the first iterator <code>i</code>
     * in the range <code>[first,last)</code> such that for any
     * iterator <code>j</code> in the range <code>[first,last)</code> 
     * the following condition holds: 
     * <code>pred(j.get(), i.get()) == false</code>.
     * Exactly <code>max(N-1, 0)</code> comparisons are performed.
     * @param first The beginning of the range.
     * @param last The end of the range.
     * @param pred The predicate used when testing.
     *
     */
    public static ForwardIterator min_element(ForwardIterator first, ForwardIterator last, Predicate2 pred) {
	first = Iter.copy(first);
	if (first.cmp(last))
	    return first;
	ForwardIterator result = Iter.copy(first);
	while (!first.next().cmp(last))
	    if (pred.compare(first.get(), result.get()))
		result.setTo(first);
	return result;
    }

    /**
     * Compares two sequences lexicographically and return true if
     * range1 is less than range2.
     * <code>lexicographical_compare</code> return <code>true</code> if the 
     * sequence of elements defined by the range <code>[first1,last1)</code> is
     * if lexicographically less than the sequence of elements defined 
     * by the range <code>[first2,last2)</code>.
     * it returns <code>false</code> otherwise.
     * At most <code>2 * min(N1, N2)</code> 
     * comparisons are performed.
     * @param first1 The beginning of the first range.
     * @param last1 The end of the first range.
     * @param first2 The beginning of the second range.
     * @param last2 The end of the second range.
     * @param pred The predicate used when testing.
     */
    public static boolean lexicographical_compare(InputIterator first1, InputIterator last1, InputIterator first2, InputIterator last2, Predicate2 pred) {
	first1 = Iter.copy(first1);
	first2 = Iter.copy(first2);

	while (!first1.cmp(last1) && !first2.cmp(last2)) {
	    if (pred.compare(first1.get(), first2.get()))
		return true;
	    if (pred.compare(first2.get(), first1.get()))
		return false;
	    first1.next();
	    first2.next();
	}
	return first1.cmp(last1) && first2.cmp(last2);
    }


    /**
     * Permutates the sequence into its successor.
     * <code>next_permutation</code> takes a sequence defined by the range
     * <code>[first,last)</code> and transform it into the <i>next</i> 
     * permutation.
     * The next permutation if found by assuming that set set of all permutation
     * is lexicographiclly sorted with respect to <code>pred</code>. 
     * If such a permutation exist, it returns <code>true</code>.
     * Otherwise, it transform the sequence into the smallest permutation, 
     * that is, the ascending sorted one, and returns <code>false</code>.
     * At most <code>N/2</code> swaps are performed.
     * @param first The beginning of the range.
     * @param last The end of the range.
     * @param pred The predicate used when testing.
     */
    public static boolean next_permutation(BidirectionalIterator first, BidirectionalIterator last, Predicate2 pred) {
	if (first.cmp(last))
	    return false;
	BidirectionalIterator i = Iter.copy(first);
	i.next();
	if (i.cmp(last))
	    return false;
	i = Iter.copy(last);
	i.prev();

	for (;;) {
	    BidirectionalIterator ii = Iter.copy(i);
	    i.prev();
	    if (pred.compare(i.get(), ii.get())) {
		BidirectionalIterator j = Iter.copy(last);
		j.prev();
		while (!pred.compare(i.get(), j.get()))
		    j.prev();
		iter_swap(i, j);
		reverse(ii, last);
		return true;
	    }
	    if (i.cmp(first)) {
		reverse(first, last);
		return false;
	    }
	}
    }

    /**
     * Permutates the sequence into its predecessor.
     * <code>prev_permutation</code> takes a sequence defined by the range
     * <code>[first,last)</code> and transform it into the <i>previous</i> 
     * permutation.
     * The previous permutation if found by assuming that set set of all 
     * permutation is lexicographiclly sorted with respect to 
     * <code>pred</code>. 
     * If such a permutation exist, it returns <code>true</code>.
     * Otherwise, it transform the sequence into the largest permutation, 
     * that is, the descending sorted one, and returns <code>false</code>.
     * At most <code>N/2</code> swaps are performed.
     * @param first The beginning of the range.
     * @param last The end of the range.
     * @param pred The predicate used when testing.
     */
    public static boolean prev_permutation(BidirectionalIterator first, BidirectionalIterator last, Predicate2 pred) {
	if (first.cmp(last))
	    return false;
	BidirectionalIterator i = Iter.copy(first);
	i.next();
	if (i.cmp(last))
	    return false;
	i = Iter.copy(last);
	i.prev();

	for (;;) {
	    BidirectionalIterator ii = Iter.copy(i);
	    i.prev();
	    if (pred.compare(ii.get(), i.get())) {
		BidirectionalIterator j = Iter.copy(last);
		j.prev();
		while (!pred.compare(j.get(), i.get()))
		    j.prev();
		iter_swap(i, j);
		reverse(ii, last);
		return true;
	    }
	    if (i.cmp(first)) {
		reverse(first, last);
		return false;
	    }
	}
    }
   
    /**
     * Accumulate over the range.
     * <code>accumulate</code> initializes the the accumulator <code>acc</code>
     * with the initial value <code>init</code> and then performing
     * <code>acc = func(acc, i.get())</code> for every iterator
     * <code>i</code> in the range <code>[first,last)</code> in order.
     * <code>func</code> is assumed not to cause side effects.
     * @param first The beginning of the range.
     * @param last The end of the range.
     * @param init The initial value.
     * @param func The binary function.
     * @return The resulting value of <code>acc</code>.
     */
    public static Object accumulate(InputIterator first, InputIterator last, Object init, Function2 func) {
	first = Iter.copy(first);
	while (!first.cmp(last)) {
	    init = func.perform(init, first.get());
	    first.next();
	}
	return init;
    }

    /**
     * Compute the inner product of two ranges.
     * <code>inner_product</code> initializes the accumulator <code>acc</code>
     * with the initial value <code>init</code> and then performing
     * <code>acc = func1(acc, func2(i1.get(), i2.get()))</code> 
     * for every iterator <code>i1</code> in the range 
     * <code>[first1,last1)</code> and every iterator <code>i2</code> 
     * in the range <code>[first2,last2)</code> in order.
     * <code>func1</code> and <code>func2</code> is assumed not to cause 
     * side effects.
     * @param first1 The beginning of the first range.
     * @param last1 The end of the first range.
     * @param first2 The beginning of the second range.
     * @param last2 The end of the second range.
     * @param init The initial value.
     * @param func1 The binary function used for accumulation.
     * @param func2 The binary function used for multiplying.
     * @return The resulting value of <code>acc</code>.
     */
    public static Object inner_product(InputIterator first1, InputIterator last1, InputIterator first2, Object init, Function2 func1, Function2 func2) {
	first1 = Iter.copy(first1);
	first2 = Iter.copy(first2);
	while (!first1.cmp(last1)) {
	    init = func1.perform(init, func2.perform(first1.get(), first2.get()));
	    first1.next();
	    first2.next();
	}
	return init;
    }

    /**
     * Computes the partial sum of the elements in the range.
     * <code>partial_sum</code> assigns to every iterator <code>i</code>
     * in the range <code>[result,...)</code> a value equal to <code>
     * func2(func2(..., func2(first.get(), first.get(1)), ... ), first.get(i-result)))
     * </code>. 
     * <code>partial_sum</code> return the end of the result range.
     * Exactly <code>N-1</code> applications of func are performed.
     * <code>func</code> is assumed not to cause side effects.
     * <code>result</code> may be equal to <code>first</code>.
     * @param first The beginning of the range.
     * @param last The end of the range.
     * @param result The resulting range.
     * @param func The binary function.
     * @return The end of the resulting range.
     */
    public static Object partial_sum(InputIterator first, InputIterator last, OutputIterator result, Function2 func) {
	if (first.cmp(last))
	    return result;
	first = Iter.copy(first);
	result = Iter.copy(result);
	result.put(first.get());
	Object value = first.get();
	while (!first.next().cmp(last)) {
	    value = func.perform(value, first.get());
	    result.next();
	    result.put(value);
	}
	result.next();
	return result;
    }



    /**
     * Computes the adjancent difference of the elements in the range.
     * <code>adjancent_difference</code> assigns to every element 
     * element referred to be iterator <code>i</code> 
     * in the range <code>[result+1,...)</code> a value equal to 
     * <code>func(first.get(i-result), first.get(i-result-1))</code>. 
     * <code>result</code> gets the value of <code>first.get()</code>.
     * <code>adjancent_difference</code> returns the end of the result range.
     * Exactly <code>N-1</code> applications of func are performed.
     * <code>func</code> is assumed not to cause side effects.
     * <code>result</code> may be equal to <code>first</code>.
     * @param first The beginning of the range.
     * @param last The end of the range.
     * @param result The resulting range.
     * @param func The binary function.
     * @return The end of the resulting range.
     */
    public static OutputIterator adjacent_difference(InputIterator first, InputIterator last, OutputIterator result, Function2 func) {
	if (first.cmp(last))
	    return result;
	result.put(first.get());
	return __adjacent_difference(first, last, result, func);
    }

    /**
     * 
     */
    private static OutputIterator __adjacent_difference(InputIterator first, InputIterator last, OutputIterator result, Function2 func) {
	first = Iter.copy(first);
	result = Iter.copy(result);

	Object value = first.get();
	while (!first.next().cmp(last)) {
	    Object tmp = first.get();
	    result.next();
	    result.put(func.perform(tmp, value));
	    value = tmp;
	}
	result.next();
	return result;
    }

    /**
     * XXX.
     */
    public static void iota(ForwardIterator first, ForwardIterator last, Object value, Function2 func, Object incr) {
	while (!first.cmp(last)) {
	    first.put(value);
	    value = func.perform(value, incr);
	    first.next();
	}
    }

    /**
     * 
     */
    private static Object[] _get_temp_buffer(int len) {
	return new Object[min(len, tmp_buffer_size)];
    }
    
}
