package sjl;

/**
 * The <code>Array</code> class contains static functions that makes 
 * it possible to use the java builtin arrays in combination with the 
 * algorithms.
 * The functions below will return random iterators for the beginning and 
 * past-the-end for argument array. </br>
 * This example shows how the array iterator adapters can be used with the
 * generic algorithms.
 * <pre>
 *      String array[] = new String[20];
 *      Vector vector(20);
 *
 *      // fill the array with strings.
 *      ...
 *
 *      // Copy the contents of the array into the vector.
 *      Algo.copy(Array.begin(array), Array.end(array), 
 *                            new BackInsertIterator(vector));
 *
 * </pre>
 * <p>Copyright <a href=../Copyrght.htm>&copy</a> 1996 Finn Bock<p>
 */

public class Array {

    /**
     * We do not allow instances of the Array class 
     */
    private Array() {
    }

    /**
     * Return an array iterator adapter for the beginning of the array
     */
    public static RandomIterator begin(Object[] array) {
	return new ArrayIterator(array, 0);
    }

    /**
     * Return an array iterator adapter for the end of the array
     */
    public static RandomIterator end(Object[] array) {
	return new ArrayIterator(array, array.length);
    }

    /**
     * Return an array iterator adapter for a position in the array
     */
    public static RandomIterator begin(Object[] array, int pos) {
	return new ArrayIterator(array, pos);
    }

    /**
     * Return an array iterator adapter for the beginning of the array
     */
    public static RandomIterator begin(byte[] array) {
	return new ArrayIteratorByte(array, 0);
    }

    /**
     * Return an array iterator adapter for a position in the array
     */
    public static RandomIterator begin(byte[] array, int pos) {
	return new ArrayIteratorByte(array, pos);
    }

    /**
     * Return an array iterator adapter for the end of the array
     */
    public static RandomIterator end(byte[] array) {
	return new ArrayIteratorByte(array, array.length);
    }

    /**
     * Return an array iterator adapter for the beginning of the array
     */
    public static RandomIterator begin(short[] array) {
	return new ArrayIteratorShort(array, 0);
    }

    /**
     * Return an array iterator adapter for a position in the array
     */
    public static RandomIterator begin(short[] array, int pos) {
	return new ArrayIteratorShort(array, pos);
    }

    /**
     * Return an array iterator adapter for the end of the array
     */
    public static RandomIterator end(short[] array) {
	return new ArrayIteratorShort(array, array.length);
    }

    /**
     * Return an array iterator adapter for the beginning of the array
     */
    public static RandomIterator begin(int[] array) {
	return new ArrayIteratorInteger(array, 0);
    }

    /**
     * Return an array iterator adapter for a position in the array
     */
    public static RandomIterator begin(int[] array, int pos) {
	return new ArrayIteratorInteger(array, pos);
    }

    /**
     * Return an array iterator adapter for the end of the array
     */
    public static RandomIterator end(int[] array) {
	return new ArrayIteratorInteger(array, array.length);
    }

    /**
     * Return an array iterator adapter for the beginning of the array
     */
    public static RandomIterator begin(float[] array) {
	return new ArrayIteratorFloat(array, 0);
    }

    /**
     * Return an array iterator adapter for a position in the array
     */
    public static RandomIterator begin(float[] array, int pos) {
	return new ArrayIteratorFloat(array, pos);
    }

    /**
     * Return an array iterator adapter for the end of the array
     */
    public static RandomIterator end(float[] array) {
	return new ArrayIteratorFloat(array, array.length);
    }

    /**
     * Return an array iterator adapter for the beginning of the array
     */
    public static RandomIterator begin(double[] array) {
	return new ArrayIteratorDouble(array, 0);
    }

    /**
     * Return an array iterator adapter for a position in the array
     */
    public static RandomIterator begin(double[] array, int pos) {
	return new ArrayIteratorDouble(array, pos);
    }

    /**
     * Return an array iterator adapter for the end of the array
     */
    public static RandomIterator end(double[] array) {
	return new ArrayIteratorDouble(array, array.length);
    }

    /**
     * Return an array iterator adapter for the beginning of the array
     */
    public static RandomIterator begin(boolean[] array) {
	return new ArrayIteratorBoolean(array, 0);
    }

    /**
     * Return an array iterator adapter for a position in the array
     */
    public static RandomIterator begin(boolean[] array, int pos) {
	return new ArrayIteratorBoolean(array, pos);
    }

    /**
     * Return an array iterator adapter for the end of the array
     */
    public static RandomIterator end(boolean[] array) {
	return new ArrayIteratorBoolean(array, array.length);
    }

    /**
     * Return an array iterator adapter for the beginning of the array
     */
    public static RandomIterator begin(char[] array) {
	return new ArrayIteratorCharacter(array, 0);
    }

    /**
     * Return an array iterator adapter for a position in the array.
     */
    public static RandomIterator begin(char[] array, int pos) {
	return new ArrayIteratorCharacter(array, pos);
    }

    /**
     * Return an array iterator adapter for the end of the array
     */
    public static RandomIterator end(char[] array) {
	return new ArrayIteratorCharacter(array, array.length);
    }

    /**
     * Return an array iterator adapter for the beginning of the array
     */
    public static RandomIterator begin(String string) {
	return new ArrayIteratorString(string, 0);
    }

    /**
     * Return an array iterator adapter for a position in the string.
     */
    public static RandomIterator begin(String string, int pos) {
	return new ArrayIteratorString(string, pos);
    }

    /**
     * Return an array iterator adapter for the end of the array
     */
    public static RandomIterator end(String string) {
	return new ArrayIteratorString(string, string.length());
    }

    /**
     * Return an array iterator adapter for the beginning of the array
     */
    public static RandomIterator begin(StringBuffer string) {
	return new ArrayIteratorStringBuffer(string, 0);
    }

    /**
     * Return an array iterator adapter for a position in the string.
     */
    public static RandomIterator begin(StringBuffer string, int pos) {
	return new ArrayIteratorStringBuffer(string, pos);
    }

    /**
     * Return an array iterator adapter for the end of the array
     */
    public static RandomIterator end(StringBuffer string) {
	return new ArrayIteratorStringBuffer(string, string.length());
    }
}
