package sjl;

/**
 * An iterator adapter that causes all assignment operations (put) to be 
 * converted into an <code>push_back()</code> operation. 
 * When the <code>BackInsertIterator</code> is constructed a container 
 * which implements the 
 * <a href=sjl.BackInsertContainer.html>BackInsertContainer</a> interface must
 * be supplied.
 * Currently the <a href=sjl.List.html>List</a>, the 
 * <a href=sjl.Deque.html>Deque</a> and the <a href=sjl.Vector.html>Vector</a>
 * implements <code>BackInsertContainer</code>.
 * 
 * <p>Copyright <a href=..\Copyrght.htm>&copy</a> 1996 Finn Bock<p>
 * 
 * @see List 
 * @see Deque
 * @see Vector
 */
public class BackInsertIterator extends Iter implements OutputIterator {
    BackInsertContainer container;

    /**
     * Construct an insert iterator for the container, which convert all
     * assigments (put) into the <code>push_back</code> method. 
     * The container must be a <code>BackInsertContainer</code>, that is, 
     * it must support the <code>push_back</code> method.
     */
    public BackInsertIterator(BackInsertContainer container) {
	this.container = container;
    }

    /**
     * Insert object at the end of the container.
     * @param object The object that will be inserted in the container
     * @return object is returned.
     */
    public Object put(Object object)  {
	container.push_back(object);
	return object;
    }

    /**
     * Do nothing. All assignment to this iterator will always be at the
     * back of the container, so advancing the iterator is ignored.
     */
    public Iterator next() {
	return this;
    }

    /**
     * Invalid. This method is inherited from Iterator, but should
     * not exist in an OutputIterator.
     * XXX: Throw an exception perhaps??.
     */
    public boolean cmp(Iterator i) {
	return false;
    }

    /**
     * Return this. Since insert iterators does not maintain any position,
     * there is no need for actually clone the iterator. 
     */
    public Iterator genericCopy() {
	return this;
    }

    /**
     * Assign this iterator to another container. This is faster than creating
     * a new BackInsertIterator object.
     */
    public void setTo(Iterator i) {
        BackInsertIterator ii = (BackInsertIterator)i;
	container = ii.container;
    }

}



