package sjl;

/**
 * <code>Deque</code> is a kind of sequence that, like vector, supports 
 * random access iterators.
 * In additition it supports constant time insert and erase operations at 
 * the beginning or the end; insert and erase in the middle take linear time. 
 * As with vectors, storage management is handled automatically.
 * <p>
 * <code>insert</code> and <code>push</code> invalidates all the iterators
 * and references to the deque. 
 * In the worst case, inserting a single element into a deque takes time 
 * linear in the minimum of the distance from the insertion point to the 
 * beginning of the deque and the distance from the insertion point to 
 * the end of the deque.
 * Inserting a single element either at the beginning or at the end  of a 
 * deque always takes constant time. That is, a deque is especially 
 * optimized for pushing and popping elements at the beginning and end.
 * <p>
 * 
 * <code>erase</code> and <code>pop</code> invalidates all the iterators 
 * and references to deque.
 *
 * <p>Copyright <a href=..\Copyrght.htm>&copy</a> 1996 Finn Bock<p>
 */ 

public class Deque implements ReversibleContainer, QueueContainer {
    /**
     * The size of the first level array. All new Deque containers are created
     * with an initial map with a size of <code>MapSize</code>. The default
     * value is 256.
     */
    static public int MapSize = 256;
    /**
     * The size of the second level buffer. All new Deque containers are 
     * created with buffers with a size of <code>BufferSize</code>. The default
     * value is 256.
     */
    static public int BufferSize = 256;

    /**
     * Private. The actual data array.
     */
    protected Object map[][];

    /**
     * Private. The actual map size.
     */
    protected int map_size;
    /**
     * Private. The actual buffer size.
     */
    protected int buffer_size;

    private DequeIterator start;
    private DequeIterator finish;
    private int length;

    /**
     * Construct a new empty Deque.
     */
    public Deque() {
	map = null;
	start = new DequeIterator(this, 0, 0);
	finish = new DequeIterator(this, 0, 0);
        length = 0;
        map_size = Deque.MapSize;
        buffer_size = Deque.BufferSize;
    }

    /**
     * Construct a new Deque with an initial size of <code>size</code> elements,
     * each element initialized to with a reference to value.
     * @param size The size of the new deque.
     * @param value The initial value of each element in the deque.
     */
    public Deque(int size, Object value) {
	this();
        insert(beginRef(), size, value);
    }

    /**
     * Construct a new Deque with the size and contents of another deque.
     * @param deque The deque which is copied into the new deque.
     */
    public Deque(Deque deque) {
	this();
	Algo.copy(deque.begin(), deque.end(), new BackInsertIterator(this));
    }

    /**
     * Construct a new Deque with the contents of the 
     * range <code>[first,last)</code>.
     * @param first The beginning of the range.
     * @param last The end of the range.
     */
    public Deque(ForwardIterator first, ForwardIterator last) {
	this();
	Algo.copy(first, last, new BackInsertIterator(this));
    }

    /**
     * Erase all the elements in the container.
     */
    public void flush() {
	map = null;
	start._set(0, 0);
	finish._set(0, 0);
        length = 0;
    }

    /**
     * Compare the elements in this container with the elements 
     * in another container.
     * @return <code>true</code> is the elements match.
     */
    public boolean equals(Object container) {
	if ((container != null) && (container instanceof Container)) {
	    Container c = (Container)container;
	    return size() == c.size() &&  Algo.equal(beginRef(), endRef(),
			(InputIterator)c.beginGeneric());
	}
	return false;
    }

    /**
     * Returns the iterator that represents the beginning of the deque.
     */
    public DequeIterator begin() {
	return new DequeIterator(start);
    }

    /**
     * Returns a reference to the iterator that represents the beginning of 
     * the deque.
     */
    public DequeIterator beginRef() {
	return start;
    }

    /**
     * Returns the iterator that represents the end of the deque.
     */
    public DequeIterator end() {
	return new DequeIterator(finish);
    }

    /**
     * Returns a reference to the iterator that represents the end of 
     * the deque.
     */
    public DequeIterator endRef() {
	return finish;
    }

    /**
     * Returns the iterator that represents the beginning of the deque as
     * an untyped iterator.
     */
    public ForwardIterator beginGeneric() {
	return start;
    }

    /**
     * Returns the iterator that represents the end of the deque as
     * an untyped iterator.
     */
    public ForwardIterator endGeneric() {
	return finish;
    }

    /**
     * Returns a copy of the reverse iterator that represents the 
     * beginning (end) of the list.
     */
    public ReverseRandomIterator rbegin() {
	return new ReverseRandomIterator(finish);
    }

    /**
     * Returns a copy of the iterator that represents the end (beginning)
     * of the list.
     */
    public ReverseRandomIterator rend() {
	return new ReverseRandomIterator(start);
    }

    /**
     * Returns a copy of the reverse iterator that represents the 
     * beginning (end) of the list as an untyped Iterator.
     */
    public Iterator rbeginGeneric() {
	return rbegin();
    }

    /**
     * Returns a copy of the iterator that represents the end (beginning)
     * of the list as an untyped Iterator.
     */
    public Iterator rendGeneric() {
	return rend();
    }

    /**
     * Returns the number of elements stored in the deque.
     */
    public int size() {
	return length;
    }

    /**
     * Returns the maximum number of elements that can be stored in a deque.
     */
    public int max_size() {
	return Integer.MAX_VALUE;
    }

    /**
     * Returns <code>true</code> if the deque does not contain any elements.
     */
    public boolean empty() {
	return length == 0;
    }

    /**
     * Returns the <code>n</code>th element in the container.
     */
    public Object get(int n) {
	return start.get(n);
    }

    /**
     * Set the <code>n</code>th element in the container to o.
     */
    public Object put(int n, Object o) {
	return start.put(n, o);
    }

    /**
     * Return the first element in the container.
     */
    public Object front() {
	return start.get();
    }

    /**
     * Return the last element in the container.
     */
    public Object back() {
	finish.prev();
        Object o = finish.get();
	finish.next();
	return o;
    }

    /**
     * Add an element to the front of the deque.
     */
    public void push_front(Object o) {
	if (empty() || start.current == 0)
	    allocate_at_begin();
	--start.current;
 	start.put(o);
	++length;
	if (finish.current == buffer_size)
	    allocate_at_end();
    }

    /**
     * Add an element to the end of the deque.
     */
    public void push_back(Object o) {
	if (empty())
	    allocate_at_end();
 	finish.put(o);
	++finish.current;
	++length;
	if (finish.current == buffer_size)
	    allocate_at_end();
    }

    /**
     * Removes the first element in deque.
     */
    public void pop_front() {
	start.put(null);
	++start.current;
	--length;
	if (empty() || start.current == buffer_size)
	    deallocate_at_begin();
    }

    /**
     * Removes the last element in deque.
     */
    public void pop_back() {
	if (finish.current == 0)
	    deallocate_at_end();
	--finish.current;
	finish.put(null);
	--length;
	if (empty())
	    deallocate_at_end();
    }

    /**
     * Insert an element into the deque at the specified position.
     * Elements located after the position are moved.
     * @param position insert the element at this position.
     * @param o the element to insert.
     * @return An iterator to the position where the element was inserted.
     */
    public Iterator insert(Iterator position, Object o) {
	DequeIterator p = (DequeIterator)position;
	if (p.cmp(start)) {
	    push_front(o);
	    return begin();
	} else if (p.cmp(finish)) {
	    push_back(o);
	    return end().prev();
	} else {
	    int index = start.distance(p);
	    if (index < length) {
		push_front(front());
		Algo.copy(begin().next(2), begin().next(index + 1),
			  begin().next(1));
	    } else {
		push_back(back());
		Algo.copy_backward(begin().next(index), end().next(-2),
			           end().next(-1));
	    }
	    put(index, o);
	    return begin().next(index);
	}
    }

    /**
     * Insert n references to element into the deque at the specified position.
     * Elements located after the position are moved.
     * @param position insert the element at this position.
     * @param n the number of elements to insert.
     * @param o the element to insert.
     */
    public void insert(Iterator position, int n, Object o) {
	int index = start.distance(position);
	int remainder = length - index;
	if (remainder > index) {
	    if (n > index) {
		int m = n - index;
		while (m-- > 0)
		    push_front(o);
		int i = index;
		while (i-- != 0)
		    push_front(get(n - 1));
		Algo.fill(begin().next(n), begin().next(n + index), o);
	    } else {
		int i = n;
		while (i-- != 0)
		    push_front(get(n - 1));
		Algo.copy(begin().next(n + n), begin().next(n + index),
                          begin().next(n));
		Algo.fill(begin().next(index), begin().next(n + index), o);
	    }
	} else {
	    int orig_len = index + remainder;
	    if (n > remainder) {
		int m = n - remainder;
		while (m-- > 0)
		    push_back(o);
		int i = 0;
		while (i < remainder) 
		    push_back(get(index + i++));
		Algo.fill(begin().next(index), begin().next(orig_len), o);
	    } else {
		int i = 0;
		while (i < n)
		    push_back(get(orig_len - n + i++));
		Algo.copy_backward(begin().next(index),
			           begin().next(orig_len - n),
                                   begin().next(orig_len));
		Algo.fill(begin().next(index), begin().next(n + index), o);
	    }
	} 
    }

    /**
     * The elements in the range <code>[first,last)</code> are inserted 
     * into deque at the specified position.
     * Elements located after the position are moved.
     * @param position insert the element at this position.
     * @param first the beginning of the range.
     * @param last the end of the range.
     */
    public void insert(Iterator position, InputIterator first, InputIterator last) {
	RandomIterator f = (RandomIterator)first;
	RandomIterator l = (RandomIterator)last;

	int index = start.distance(position);
	int remainder = length - index;
	int n = f.distance(l);
	if (remainder > index) {
	    if (n > index) {
		DequeIterator m = (DequeIterator)Iter.copy(l).prev(index);
		while (!m.cmp(first)) {
		    m.prev();
		    push_front(m.get());
		}
		int i = index;
		while (i-- != 0)
		    push_front(get(n - 1));
		Algo.copy(Iter.copy(l).prev(index),
			  Iter.copy(l).next(n + index), 
			  begin().next(index));
	    } else {
		int i = n;
		while (i-- != 0)
		    push_front(get(n - 1));
		Algo.copy(begin().next(n + n), begin().next(n + index),
                          begin().next(n));
		Algo.copy(first, last, begin().next(index));
	    }
	} else {
	    int orig_len = index + remainder;
	    if (n > remainder) {
		DequeIterator m = (DequeIterator)Iter.copy(f).next(remainder);
		while (!m.cmp(last)) {
		    push_front(m.get());
		    m.next();
		}
		int i = 0;
		while (i < remainder) 
		    push_back(get(index + i++));
		Algo.copy(first, Iter.copy(f).next(remainder),
			  begin().next(index));
	    } else {
		int i = 0;
		while (i < n)
		    push_back(get(orig_len - n + i++));
		Algo.copy_backward(begin().next(index),
			  begin().next(orig_len - n),
                          begin().next(orig_len));
		Algo.copy(first, last, begin().next(index));
	    }
	} 
    }

    /**
     * Removes the element specified by position.
     */
    public void erase(Iterator position) {
	DequeIterator p = (DequeIterator)position;
	if (p.distance(finish) > start.distance(p)) {
	    Algo.copy_backward(start, p, Iter.copy(p).next(1));
	    pop_front();
	} else {
	    Algo.copy_backward(Iter.copy(p).next(1), finish, p);
	    pop_back();
	}
    }

    /**
     * Removes the elements in the specified range.
     * @param first the beginning of the range.
     * @param last the end of the range.
     */
    public void erase(Iterator first, Iterator last) {
	DequeIterator f = (DequeIterator)first;
	DequeIterator l = (DequeIterator)last;

	int n = f.distance(l);
	
	if (l.distance(finish) > start.distance(f)) {
	    Algo.copy_backward(start, f, l); 
	    while (n-- > 0)
	        pop_front();
	} else {
	    Algo.copy_backward(l, finish, f);
	    while (n-- > 0)
	        pop_back();
	}
    }

    /**
     * Returns the string representation of this deque.
     */
    public String toString()  {
        StringBuffer buf = new StringBuffer();
        buf.append(start);
        buf.append(finish);
        return buf.toString();
    }


    private void allocate_at_begin() {
	Object[] p = new Object[buffer_size];
	if (!empty()) {
	    if (start.node == 0) {
		int i = finish.node - start.node;
		int map_size = (i + 1) * 2;
		Object[][] tmp = new Object[map_size][];
		System.arraycopy(map, start.node, tmp, map_size / 4 + 1, i + 1);
		map = tmp;
		map[map_size / 4] = p;
		start._set(buffer_size, map_size / 4);
		finish._set(finish.current, map_size / 4 + i + 1);
	    } else {
		map[--start.node] = p;
		start._set(buffer_size, start.node);
	    }
	} else {
	    map = new Object[map_size][];
	    map[map_size / 2] = p;
	    start._set(buffer_size / 2 + 1, map_size / 2);
	    finish.setTo(start);
	}
    }


    private void allocate_at_end() {
	Object[] p = new Object[buffer_size];
	if (!empty()) {
	    if (finish.node == map.length - 1) {
		int i = finish.node - start.node;
		map_size = (i + 1) * 2;
		Object[][] tmp = new Object[map_size][];
		System.arraycopy(map, start.node, tmp, map_size / 4, i + 1);
		map = tmp;
		map[map_size / 4 + i + 1] = p;
		start._set(start.current, map_size / 4);
		finish._set(0, map_size / 4 + i + 1);
	    } else {
		map[++finish.node] = p;
		finish._set(0, finish.node);
	    }
	} else {
	    map = new Object[map_size][];
	    map[map_size / 2] = p;
	    start._set(buffer_size / 2, map_size / 2);
	    finish.setTo(start);
	}
    }

    private void deallocate_at_begin() {
	map[start.node++] = null;
	if (empty()) {
	    if (finish.current == 0)
		map[start.node] = null;
	    start._set(0, 0);
	    finish.setTo(start);
	    map = null;
	} else
	    start._set(0, start.node);
    }

    private void deallocate_at_end() {
	map[finish.node--] = null;
	if (empty()) {
	    start._set(0, 0);
	    finish.setTo(start);
	    map = null;
	} else
	    start._set(buffer_size, finish.node);
    }
}
