package sjl;

/**
 * An Iterator adapter that causes all assignment operations (put) to be 
 * converted into an <code>push_front()</code> operation. 
 * When the <code>FrontInsertIterator</code> is constructed a container 
 * which implements the 
 * <a href=sjl.FrontInsertContainer.html>FrontInsertContainer</a> interface 
 * must be supplied.
 * Currently the <a href=sjl.List.html>List</a> and the 
 * <a href=sjl.Deque.html>Deque</a> implements <code>FrontInsertIterator</code>.
 *
 * <p>Copyright <a href=..\Copyrght.htm>&copy</a> 1996 Finn Bock<p>
 *
 * @see List 
 * @see Deque
 */
public class FrontInsertIterator extends Iter implements OutputIterator {
    FrontInsertContainer container;

    /**
     * Construct an insert iterator for the container, which convert all
     * assigments (put) into the push_front method. The container must
     * be a FrontInsertContainer, that is, it must support the push_front 
     * method.
     */
    public FrontInsertIterator(FrontInsertContainer container) {
	this.container = container;
    }

    /**
     * Insert object at the front of the container.
     * @param object The object that will be inserted in the container
     * @return object is returned.
     */
    public Object put(Object object)  {
	container.push_front(object);
	return object;
    }

    /**
     * Do nothing. All assignment to this iterator will always be at the
     * front of the container, so advancing the iterator is ignored.
     */
    public Iterator next() {
	return this;
    }

    /**
     * Invalid. This method is inherited from Iterator, but should
     * not exist in an OutputIterator.
     * XXX: Throw an exception perhaps??.
     */
    public boolean cmp(Iterator i) {
	return false;
    }

    /**
     * Return this. Since insert iterators does not maintain any position,
     * there is no need for actually clone the iterator. 
     */
    public Iterator genericCopy() {
	return this;
    }

    /**
     * Assign this iterator to another container. This is faster than creating
     * a new FrontInsertIterator object.
     */
    public void setTo(Iterator i) {
        FrontInsertIterator ii = (FrontInsertIterator)i;
	container = ii.container;
    }

}
