package sjl;
import java.util.Dictionary;
import java.util.Enumeration;

/**
 * <code>HashMap</code> is a kind of associative container that supports 
 * unique keys (contains at most one of each key value) and provides 
 * for fast retrieval of the keys themselves.
 *
 * <p>Copyright <a href=..\Copyrght.htm>&copy</a> 1996 Finn Bock<p>
 */

public class HashMap extends Dictionary implements AssociativeContainer {
    Hashtbl ht;
    HashtblIterator end;
    HashtblIterator begin;
    HashtblIterator tmpiterator;

    /**
     * Construct an empty map. 
     */
    public HashMap() {
	ht = new Hashtbl(1009, new HashFunction(), new PredEqual(), new Select1st(), false);
    }

    /**
     * Construct a map and initialize it with the contents of another map. 
     * @param map Initialize the new map with the elements of map.
     */
/*
    public HashMap(HashMap map) {
	ht = new Hashtbl(map.ht, false);
    }
*/

    /**
     * Construct a map and initialize it with the elements in the 
     * range <code>[first,last)</code>.
     * @param first The beginning of the range.
     * @param last The end of the range.
     */
    public HashMap(InputIterator first, InputIterator last) {
	this();
	for (InputIterator i = Iter.copy(first); !i.cmp(last); i.next())
	    ht.insert(i.get());
    }

    /**
     * Erase all the elements in the container. 
     */
    public void flush() {
//	this();
    }

    /**
     * Compare the elements in this container with the elements 
     * in another container.
     * @return <code>true</code> is the elements match.
     */
    public boolean equals(Object container) {
	if ((container != null) && (container instanceof Container)) {
	    Container c = (Container)container;
	    return size() == c.size() &&  Algo.equal(beginRef(), endRef(),
			(InputIterator)c.beginGeneric());
	}
	return false;
    }

    /**
     * Returns the iterator that represents the beginning of the map. 
     */
    public ForwardIterator begin() {
	return ht.begin();
    }

    /**
     * Returns a reference to the iterator that represents the beginning 
     * of the map. 
     */
    public ForwardIterator beginRef() {
	return ht.begin();
    }

    /**
     * Returns the iterator that represents the end of the map. 
     */
    public ForwardIterator end() {
	return ht.end();
    }

    /**
     * Returns a reference to the iterator that represents the end of the map. 
     */
    public ForwardIterator endRef() {
	return ht.end();
    }

    /**
     * Returns the iterator that represents the beginning of the map as an
     * untyped iterator. 
     */
    public ForwardIterator beginGeneric() {
	return ht.begin();
    }

    /**
     * Returns the iterator that represents the end of the map as an
     * untyped iterator. 
     */
    public ForwardIterator endGeneric()  {
	return ht.end();
    }

    /**
     * Returns the number of elements in the map.
     */
    public int size() {
	return ht.size();
    }

    /**
     * Returns the maximum number of elements allowed in a map.
     */
    public int max_size() {
	return Integer.MAX_VALUE;
    }

    /**
     * Returns <code>true</code> if the map does not contain any elements.
     */
    public boolean empty() {
	return ht.size() == 0;
    }

    /**
     * Insert an element into the map. The element is only inserted if 
     * there is no element in the container with a key equal to the key 
     * of value.
     * @param value The element to insert.
     * @return The first component of pair is an iterator that point to the 
     * element with a key equal to the key of value. The second component of 
     * pair is a Boolean that is true if value was actually inserted, and 
     * false if the container already contained a element equal to value.
     */
    public Pair insert(Pair value) {
	return ht.insert(value);
    }

    /**
     * Insert the elements from the range <code>[first,last)</code> into 
     * the map.
     * @param first The beginning of the range.
     * @param last The end of the range.
     */
    public void insert(InputIterator first, InputIterator last) {
	for (InputIterator i = Iter.copy(first); !i.cmp(last); i.next()) 
	    ht.insert(i.get());
    }

    /**
     * Erase the element pointed to by the iterator.
     * @param position Points to the element to be removed.
     */
    public void erase(Iterator position) {
	ht.erase((HashtblIterator)position);
    }

    /**
     * Erase the element with key equal to value.
     * @return The number of element erased.
     */
    public int erase(Object value) {
	return ht.erase(value);
    }

    /**
     * Erase the elements in the range <code>[first,last)</code>.
     * @param first The beginning of the range.
     * @param last The end of the range.
     */
    public void erase(Iterator first, Iterator last) {
	ht.erase((HashtblIterator)first, (HashtblIterator)last);
    }

    /**
     * Search the map for an element with a key equal to value.
     * @param value The key to search for.
     * @return An Iterator pointing to the element with a key equal to value,
     *          or end() if such an element is not found.
     */
    public ForwardIterator find(Object value) {
	return ht.find(value);
    }

    /**
     * Retrives the value associated with the key.
     * @param key The key to search for.
     * @return The value object associated with the key, or null of the
     *          key does not exist in the map.
     */
    public Object get(Object key) {
        ForwardIterator i = ht.find(key);
        if (i.cmp(end()))
            return null;
        return ((Pair)i.get()).second;
    }

    /**
     * Insert the key-value objects into the map.
     * The objects is inserted into the map as a pair.
     * If the map already contains such a key, the value specified
     * replces the existing value.
     * @param key The key to search for.
     * @param key The value to be inserted.
     */
    public Object put(Object key, Object value) {
        ForwardIterator i = ht.find(key);
        Object old;
        if (i.cmp(end())) {
            old = null;
            insert(new Pair(key, value));
        } else {
            old = ((Pair)i.get()).second;
            ((Pair)i.get()).second = value;
        }
        return old;
    }


    /**
     * Returns the number of element with a key equal to value.
     */
    public int count(Object value) {
	return ht.count(value);
    }


    /**
     * Returns a pair like 
     * <code>Pair(lower_bound(value), upper_bound(value))</code>.
     */
    public Pair equal_range(Object value) {
	return ht.equal_range(value);
    }

    /**
     * From Dictionary; Returns true if the map contains no elements.
     * @return true of the Map contains no elements.
     */
    public boolean isEmpty() {
        return empty();
    }

    /**
     * From Dictionary; Returns an enumeration of the Map's keys.
     */
    public Enumeration keys() {
        return new MapEnumerator(beginRef(), endRef(), true);
    }

    /**
     * From Dictionary; Returns an enumeration of the elements.
     * Use the Enumeration methods on the returned object to fetch
     * the elements sequentially.
     */
    public Enumeration elements() {
        return new MapEnumerator(beginRef(), endRef(), false);
    }


    /**
     * From Dictionary; Removes the element corresponding to the key.
     * Does nothing if not present.
     * @param key the key that needs to be removed.
     * @return the value of the key, or null if the key was not found.
     */
    public Object remove(Object key) {
        Object val = get(key);
        erase(key);
        return val;
    }

    private void dump() {
	ht.dump();
    }
}
