package sjl;
import java.io.*;

/**
 * To make it possible for the generic algorihms to work directly with
 * input streams, the <code>InputStreamIterator</code> have been created. <br>
 * Ex:
 * <pre>
 *     fin = new FileInputStream("inputfile");
 *     InputStreamIterator is = new InputStreamIterator(fin, new ReadIntegerText());
 *     InputStreamIterator ie = new InputStreamIterator();
 *     Algo.copy(is, ie, new BackInsertIterator(vector));
 * </pre>
 * The <code>InputStreamIterator</code> reads (using the 
 * <code>ReadIntegerText</code> function object) successive elements from 
 * the input stream (fin) for which it was contructed.
 * When the <code>InputStreamIterator</code> is contructed, and every 
 * time <code>next()</code> is used, the iterator reads and stores a value. 
 * If the end of the stream is reached (the stream throws a EOFException), 
 * the iterator becomes equal to the iterator <i>end-of-stream</i> 
 * iterator value. 
 * The constructor with no arguments always contructs an end of stream iterator
 * object which is the only legitimate iterator to be used for the end 
 * condition. 
 * <p>
 * It is impossible to store things into a <code>InputStreamIterator</code>. 
 * <p>
 * The main peculiarity of the <code>InputStreamIterator<code> is the fact 
 * that the <code>next()</code> method is not equality preservings, 
 * that is <code>i.cmp(j)</code> does not guarantee at all that 
 * <code>i.next(); j.next(); i.cmp(j)</code>. 
 * Every time <code>next()</code> is used a new value is returned. 
 * <p>
 * The practical consequence of this fact is that 
 * <code>InputStreamIterators<code> can be used only for one-pass algorithms, 
 * which actually makes perfect sense, since for multi-pass algorithms 
 * it is always more appropriate to use inmemory data structures.
 * <p>
 * Two end-of-stream iterators are always equal. 
 * An end-of-stream iterator is not equal to a non-end-of-stream iterator. 
 *
 * <p>Copyright <a href=../Copyrght.htm>&copy</a> 1996 Finn Bock<p>
 */ 
public class InputStreamIterator extends Iter implements InputIterator {
    DataInputStream stream;
    Object value;
    Function1 reader;
    boolean end_marker;

    /** 
     * Construct a InputStreamIterator for the stream using the function
     * object as a reader.
     * @param istream The iterator is constructed for this stream. A 
     *                DataInputStream is constructed for istream.
     * @param func The function object used as a reader.
     * @see DataInputStream
     */
    public InputStreamIterator(InputStream istream, Function1 func) {
	init(new DataInputStream(istream), func);
    }

    /** 
     * Construct a InputStreamIterator for the stream using the function
     * object as a reader.
     * @param istream The iterator is constructed for this stream. 
     */
    public InputStreamIterator(DataInputStream istream, Function1 func) {
	init(istream, func);
    }

    /** 
     * Construct a end-of-stream InputStreamIterator.
     * <br> XXX: perhaps a static variable is better.
     */
    public InputStreamIterator() {
	end_marker = true;
    }

    /**
     * Return the current value. If called multiple times, the same value is 
     * returned.
     */
    public Object get()  {
	return value;
    }

    /**
     * Advance the iterator to the next element. A new value is read.
     */
    public Iterator next()  {
	read();
	return this;
    }

    /**
     * Compare two iterators.
     * @return true if both iterators are non-end-of-stream or end-of-stream.
     */
    public boolean cmp(Iterator iterator)  {
	InputStreamIterator i = (InputStreamIterator)iterator;
	return end_marker == i.end_marker || 
		end_marker == false && i.end_marker == false;
    }

    /**
     * XXX: Is this possible???
     * @return this
     */
    public Iterator genericCopy() {
	return this;
    }

    private void read()  {
	if (!end_marker)
	    value = reader.perform(stream);
	if (value == null)
	    end_marker = true;
    }

    private void init(DataInputStream istream, Function1 func)  {
	stream = istream;
	reader = func;
	end_marker = false;
	read();
    }

}
