package sjl;

/**
 * An iterator adapter that causes all assignment operations (put) to be 
 * converted into an <code>insert</code> operation. 
 * When the <code>InsertIterator</code> is constructed a container which 
 * implements the <a href=sjl.SequenceContainer.html>SequenceContainer</a>
 * is supplied.
 * Currently the <a href=sjl.List.html>List</a>, the 
 * <a href=sjl.Deque.html>Deque</a> and <a href=sjl.Vector.html>Vector</a> 
 * implements <code>SequenceContainer</code>.
 *
 * <p>Copyright <a href=../Copyrght.htm>&copy</a> 1996 Finn Bock<p>
 *
 * @see List 
 * @see Deque
 * @see Vector
 */
public class InsertIterator extends Iter implements OutputIterator {
    SequenceContainer container;
    Iterator iter;

    /**
     * Construct an insert iterator for the container, which convert all
     * assigments (put) into the insert method. The container must
     * be a SequenceContainer, that is, it must support the 
     * insert(Iterator, value) method.
     */
    public InsertIterator(SequenceContainer container, Iterator iter) {
	this.container = container;
	this.iter = iter;
    }

    /**
     * Insert object at the front of the container.
     * @param object The object that will be inserted in the container
     * @return object is returned.
     */
    public Object put(Object object)  {
	iter = container.insert(iter, object);
	iter.next();
	return object;
    }

    /**
     * Do nothing. All assignment to this iterator will always be at the
     * front of the container, so advancing the iterator is ignored.
     */
    public Iterator next() {
	return this;
    }

    /**
     * Invalid. This method is inherited from Iterator, but should
     * not exist in an OutputIterator.
     * XXX: Throw an exception perhaps??.
     */
    public boolean cmp(Iterator i) {
	return false;
    }

    /**
     * Return this. Since insert iterators does not maintain any position,
     * there is no need for actually clone the iterator. 
     */
    public Iterator genericCopy() {
	return this;
    }

    /**
     * Assign this iterator to another container. This is faster than creating
     * a new InsertIterator object.
     */
    public void setTo(Iterator i) {
        InsertIterator ii = (InsertIterator)i;
	container = ii.container;
	iter = ii.iter;
    }

}
