package sjl;

/**
 * <code>List</code> is a kind of sequence that supports bidirectional iterators
 * and allow constant time insert and erase operations anywhere within the 
 * sequense, with storage management handled automatically. Unlike vectors 
 * and deques, fast random access to list elements are not suported, 
 * but many algorithms only need sequential access anyway. <p>
 * 
 * <code>insert</code> does not affect the validity of iterators. 
 * Insertion of a single element into a list takes constant time. 
 * Insertion of multiple elements into a list is linear in the number of 
 * elements inserted. <p>
 * 
 * <code>erase</code> invalidates only the iterators to the erased elements. 
 * Erasing a single element is constant time. 
 * Erasing a range in a list is linear time in the size of the range.
 *
 * <p>Copyright <a href=..\Copyrght.htm>&copy</a> 1996 Finn Bock<p>
 */

public class List implements ReversibleContainer, QueueContainer {
    ListNode node;
    int length;
    ListIterator begin;
    ListIterator end;
    ListIterator tmpIter;

    /**
     * Constructs an empty list.
     */
    public List() {
	length = 0;
	node = get_node();
	node.prev = node;
	node.next = node;
	begin = new ListIterator(node.next);
	end = new ListIterator(node);
    }

    /**
     * Constructs a list with <code>n</code> element each pointing to value.
     * @param n The number of elements the list will contain.
     * @param value The object that is inserted in each in the list.
     */
    public List(int n, Object value) {
	this();
	insert(begin, n, value);
    }

    /**
     * Constructs a list with elements found in the range 
     * <code>[first,last)</code>.
     * @param first The beginning of the range.
     * @param last The end of the range.
     */
    public List(InputIterator first, InputIterator last) {
	this();
	insert(begin, first, last);
    }

    /**
     * Constructs a list with all the elements from another list.
     * @param list The elements in this list is copied into the newly 
     * contructed list.
     */
    public List(List list) {
	this();
	insert(begin, list.begin(), list.end());
    }

    /** 
     * Erase all the elements in the container.
     */
    public void flush() {
	length = 0;
	node = get_node();
	node.prev = node;
	node.next = node;
	begin._set(node.next);
	end._set(node);
    }

    /**
     * Swaps the contents of the current list with those of the input list.
     * The current list replaces x and vice versa.
     */
    public void swap(List x) {
	ListNode p1 = node; node = x.node; x.node = p1;
	int p2 = length; length = x.length; x.length = p2;
        ListIterator p3 = begin; begin = x.begin; x.begin = p3;
        ListIterator p4 = end; end = x.end; x.end = p4;
        ListIterator p5 = tmpIter; tmpIter = x.tmpIter; x.tmpIter = p3;
    }

    /**
     * Compare the elements in this container with the elements 
     * in another container.
     * @return <code>true</code> is the elements match.
     */
    public boolean equals(Object container) {
	if ((container != null) && (container instanceof Container)) {
	    Container c = (Container)container;
	    return size() == c.size() &&  Algo.equal(beginRef(), endRef(),
			(InputIterator)c.beginGeneric());
	}
	return false;
    }

    /**
     * Returns the iterator that represents the beginning of the list.
     */
    public BidirectionalIterator begin() {
	return new ListIterator(node.next);
    }

    /**
     * Returns a reference to the iterator that represents the 
     * beginning of the list.
     */
    public BidirectionalIterator beginRef() {
	begin.node = node.next;
	return begin;
    }

    /**
     * Returns the iterator that represents the end of the list.
     */
    public BidirectionalIterator end() {
	return new ListIterator(end);
    }

    /**
     * Returns a reference to the iterator that represents the end of the list.
     */
    public BidirectionalIterator endRef() {
	return end;
    }

    /**
     * Returns the iterator that represents the beginning of the list as a
     * untyped iterator.
     */
    public ForwardIterator beginGeneric() {
	begin.node = node.next;
	return begin;
    }

    /**
     * Returns the iterator that represents the end of the list as a
     * untyped iterator.
     */
    public ForwardIterator endGeneric() {
	return end;
    }

    /**
     * Returns a copy of the reverse iterator that represents the 
     * beginning (end) of the list.
     */
    public ReverseBidirectionalIterator rbegin() {
	return new ReverseBidirectionalIterator(end);
    }

    /**
     * Returns a copy of the iterator that represents the end (beginning)
     * of the list.
     */
    public ReverseBidirectionalIterator rend() {
	begin.node = node.next;
	return new ReverseBidirectionalIterator(begin);
    }

    /**
     * Returns a copy of the reverse iterator that represents the 
     * beginning (end) of the list as an untyped Iterator.
     */
    public Iterator rbeginGeneric() {
	return rbegin();
    }

    /**
     * Returns a copy of the iterator that represents the end (beginning)
     * of the list as an untyped Iterator.
     */
    public Iterator rendGeneric() {
	return rend();
    }

    /**
     * Returns the first elements in the list.
     */
    public Object front() {
	return node.next.data;
    }

    /**
     * Returns the last elements in the list.
     */
    public Object back() {
	return node.prev.data;
    }

    /**
     * Returns the number of elements stored in the list.
     */
    public int size() {
	return length;
    }

    /**
     * Returns the maximum number of elements allowed in the list. 
     */
    public int max_size() {
	return Integer.MAX_VALUE;
    }

    /**
     * Returns <code>true</code> if the list does not contain any elements.
     */
    public boolean empty() {
	return length == 0;
    }

    /**
     * Insert at the beginning of the list.
     * @param value the element to insert. 
     */
    public void push_front(Object value) {
        insert(beginRef(), value);
    }

    /**
     * Insert at the end of the list.
     * @param value the element to insert. 
     */
    public void push_back(Object value) {
	insert(endRef(), value);
    }

    /**
     * Insert the element at the specified position.
     * @param position An iterator that points to the position where 
     *                 the element is inserted. 
     * @param value The element.
     */
    public Iterator insert(Iterator position, Object value) {
	ListIterator p = (ListIterator)position;
	ListNode tmp =  get_node();
        tmp.data = value;
        tmp.next = p.node;
        tmp.prev = p.node.prev;
        p.node.prev.next = tmp;
        p.node.prev = tmp;
        length++;
	return new ListIterator(tmp);
    }

    /**
     * Insert <code>n</code> copy of the element at the specified position.
     * @param position An iterator that points to the position where 
     *                 the elements are inserted. 
     * @param n The number of elements.
     * @param value The element.
     */
    public void insert(Iterator position, int n, Object value) {
	while (n-- > 0) 
	    insert(position, value);
    }

    /**
     * Insert the element in the range <code>[first,last)</code> into the list 
     * at the specified position.
     * @param position An iterator that points to the position where 
     *                 the elements are inserted. 
     * @param first The beginning of the range
     * @param last The end of the range
     */
    public void insert(Iterator position, InputIterator first, InputIterator last) {
	first = Iter.copy(first);
	while (!first.cmp(last)) {
	    insert(position, first.get());
	    first.next();
	}
    }

    /**
     * Remove the first element in the list.
     */
    public void pop_front() {
	erase(beginRef());
    }
    

    /**
     * Remove the last element in the list.
     */
    public void pop_back() {
	erase(end().prev());
    }

    /**
     * Remove the element to which the iterator points.
     * @param position the element to which the iterator point is removed.
     */
    public void erase(Iterator position) {
	ListIterator p = (ListIterator)position;
	p.node.prev.next = p.node.next;
	p.node.next.prev = p.node.prev;
	p.node = null;
	length--;
    }

    /**
     * Remove the elements in the range <code>[first,last)</code> from the list.
     * @param first The beginning of the range.
     * @param last The end of the range.
     */
    public void erase(Iterator first, Iterator last) {
	ListIterator f = ((ListIterator)first).copy();
	while (!f.cmp(last)) {
	    erase(first);
	    f.next();
	}
    }

    /**
     * Insert the contents of another list into this list before position.
     * All elements are removed from the other list. Essentially the 
     * contents of the orther list is transferred in into this list.
     * @param position The elements are inserted before this position.
     * @param x The source list.
     */
    public void splice(Iterator position, List x) {
	if (!x.empty()) {
	    transfer(position, x.begin(), x.end());
	    length += x.length;
	    x.length = 0;
	}
    }

    /**
     * Insert the element pointed to by the iterator <code>i</code> from
     * another list into this list before a position.
     * The element is removed from the other list. 
     * @param position The elements are inserted before this position.
     * @param x The source list.
     * @param i The source element.
     */
    public void splice(Iterator position, List x, Iterator i) {
	Iterator j = Iter.copy((ListIterator)i);
	if (position.cmp(i) || position.cmp(j.next()))
	    return;
	transfer(position, i, j);
	++length;
	--x.length;
    }

    /**
     * Insert elements in the range into this list and remove the 
     * elements from the other list.
     * Elements in the range <code>[first, last)</code> is inserted into 
     * this list before position. The elements are removed from the other list.
     * @param position The elements are inserted before this position.
     * @param x The source list.
     * @param first The beginning of the range.
     * @param last The end of the range.
     */
    public void splice(Iterator position, List x, Iterator first, Iterator last) {
	if (!first.cmp(last)) {
	    if (x != this) {
		int n = ((ListIterator)first).distance(last);
		x.length -= n;
		length += n;
	    }
	    transfer(position, first, last);
	}
    }

    /**
     * Remove all elements in the list where the element is equals to the
     * value. Remove is stable, that is, the relative order of the 
     * elements that not removed is the same as their relative order in 
     * the original list.
     * @param value The value that the elements are compared with.
     */
    public void remove(Object value) {
	ForwardIterator first = begin();
	ForwardIterator last = end();
	while (!first.cmp(last)) {
	    if (first.get().equals(value)) {
		ForwardIterator tmp = Iter.copy(first);
		first.next();
		erase(tmp);
	    } else
		first.next();
	}
    }
		
    /**
     * Unique erases all but the first element from every 
     * consecutive group of equal element in the list.
     * value. Remove is stable, that is, the relative order of the 
     * elements that not removed is the same as their relative order in 
     * the original list.
     * @param value The value that the elements are compared with.
     */
    public void unique() {
	ForwardIterator first = begin();
	ForwardIterator last = end();
	if (first.cmp(last))
	    return;
	ForwardIterator next = Iter.copy(first);
	while (!next.next().cmp(last)) {
	    if (first.get().equals(next.get())) 
		erase(next);
	    else
		first.setTo(next);
	    next.setTo(first);
	}
    }

    /**
     * Merge merges the argument list into this list 
     * (both are assumed to be sorted).
     * The merge is stable, that is, for equal elements in the two lists,
     * the elements from this list always precede elements from the
     * argument list. The argument list is empty after the merge.
     * @param x The list that is merge into this list.
     */
    public void merge(List x, Predicate2 pred) {
	ForwardIterator first1 = begin();
	ForwardIterator last1 = end();
	ForwardIterator first2 = x.begin();
	ForwardIterator last2 = x.end();
	ForwardIterator next = Iter.copy(first2);

	while (!first1.cmp(last1) && !first2.cmp(last2)) {
	    if (pred.compare(first2.get(), first1.get())) {
		next.setTo(first2);
		transfer(first1, first2, next.next());
		first2.setTo(next);
	    } else
		first1.next();
	}
	if (!first2.cmp(last2))
	    transfer(last1, first2, last2);
	length += x.length;
	x.length = 0;
    }

    /**
     * Reverse reverses the oerder of the element in the list.
     */
    public void reverse() {
	if (size() < 2)
	    return;
	ForwardIterator first = begin();
	for (first.next(); !first.cmp(end); ) {
	    Iterator old = Iter.copy(first);
	    first.next();
	    transfer(begin(), old, first);
	}
    }

    /**
     * Sort sorts the list according to the compare function object. It is
     * stable, that is, the relative order of the equal elements is preserved.
     */
    public void sort(Predicate2 pred) {
	if (size() < 2)
	    return;
	List carry = new List();
	List counter[] = new List[64];
	for (int i = 0; i < 64; i++)
	    counter[i] = new List();
	int fill = 0;
	while (!empty()) {
	    carry.splice(carry.begin(), this, begin());
	    int i = 0;
	    while (i < fill && !counter[i].empty()) {
		counter[i].merge(carry, pred);
	 	carry.swap(counter[i++]);
	    }
	    carry.swap(counter[i]);
	    if (i == fill)
		++fill;
	}
	for (int i = 1; i < fill; i++)
	    counter[i].merge(counter[i-1], pred);
	swap(counter[fill - 1]);
    }

    private void transfer(Iterator position, Iterator first, Iterator last) {
	ListIterator l = (ListIterator)last;
	ListIterator f = (ListIterator)first;
	ListIterator p = (ListIterator)position;
	l.node.prev.next = p.node;
	f.node.prev.next = l.node;
	p.node.prev.next = f.node;
	ListNode tmp = p.node.prev;
	p.node.prev = l.node.prev;
	l.node.prev = f.node.prev;
	f.node.prev = tmp;
    }

    private ListNode get_node() {
	return new ListNode();
    }
}
