package sjl;

/**
 * <code>ReverseBidirectionalIterator</code> is an iterator adapter that 
 * will traverse the container in the reverse order of an ordinary
 * <code>BidirectionalIterator</code>.
 * <pre>
 *     BidirectionalIterator iter = new ReverseBidirectionalIterator(coll.end());
 *     for (; i.cmp(coll.begin()); i.next()) { ... }
 * </pre>
 * The same functionallity can be found with.
 * <pre>
 *     for (BidirectionalIterator iter = coll.rend(); i.cmp(coll.rbegin()); i.next()) { ... }
 * </pre>
 * Any <code>BidirectionalIterator</code> "iter" can be turned into a 
 * reverse iterator with:
 * <pre>
 *     Bidirectional reverse_iter = Iter.reverse(iter);
 * </pre>
 * XXX: Not true.
 *
 * <p>Copyright <a href=..\Copyrght.htm>&copy</a> 1996 Finn Bock<p>
 */
public class ReverseBidirectionalIterator extends Iter implements BidirectionalIterator {
    BidirectionalIterator iterator;

    /**
     * Contruct a ReverseBidirectionalIterator for the iterator.
     * The newly contructed iterator will have a reference to the 
     * argument iterator.
     * Each time next() and prev() is called on the 
     * ReverseBidirectionalIterator, the argument iterator is move back 
     * and forward.
     * @param iter A BidirectionalIterator.
     */
    public ReverseBidirectionalIterator(BidirectionalIterator iter) {
	iterator = iter;
    }

    /**
     * Create a copy of the this. The copy can then be moved without
     * affecting this.
     * @return A new iterator pointing to the same element as this.
     */
    public Iterator genericCopy()  {
	return new ReverseBidirectionalIterator(Iter.copy(iterator));
    }

    /**
     * Assign this iterator to another iterator. This is faster than creating
     * a new ReverseBidirectionalIterator object.
     */
    public void setTo(Iterator i) {
        ReverseBidirectionalIterator ii = (ReverseBidirectionalIterator)i;
	iterator = ii.iterator;
    }

    /**
     * Advance the iterator to the next element. Calls <code>prev()</code> 
     * on the actual iterator.
     */
    public Iterator next()  {
	return iterator.prev();
    }

    /**
     * Advance the iterator to the previous element. Calls <code>next()</code> 
     * on the actual iterator.
     */
    public Iterator prev()  {
	return iterator.next();
    }

    /**
     * Return the current value pointed to by the iterator.
     */
    public Object get()  {
	iterator.prev();
	Object o = iterator.get();
	iterator.next();
	return o;
    }

    /**
     * Places a new element in the position pointed to by the iterator.
     */
    public Object put(Object o)  {
	iterator.prev();
	iterator.put(o);
	iterator.next();
	return o;
    }

    /**
     * Compares two iterators.
     * @return true if the iterators point to the same position.
     */
    public boolean cmp(Iterator i)  {
	return iterator.cmp(i);
    }

    /**
     * Returns the distance between two iterators.
     */
    public int distance(Iterator i) {
	return ((ReverseBidirectionalIterator)i).iterator.distance(iterator);
    }
}
