package sjl;

/**
 * <code>ReverseRandomIterator</code> is an iterator adapter that will traverse 
 * the container in the reverse order of an ordinary 
 * <code>RandomlIterator</code>.
 * <pre>
 *     RandomIterator iter = new ReverseRandomIterator(coll.end());
 *     for (; i.cmp(coll.begin()); i.next()) { ... }
 * </pre>
 * The same functionallity can be found with.
 * <pre>
 *     for (RandomIterator iter = coll.rend(); i.cmp(coll.rbegin()); i.next()) { ... }
 * </pre>
 * Any <code>RandomIterator</code> "iter" can be turned into a reverse 
 * iterator with:
 * <pre>
 *     Random reverse_iter = Iter.reverse(iter);
 * </pre>
 * XXX: not true.
 *
 * <p>Copyright <a href=..\Copyrght.htm>&copy</a> 1996 Finn Bock<p>
 */
public class ReverseRandomIterator extends Iter implements RandomIterator {
    RandomIterator iterator;

    /**
     * Construct a new ReverseRandomIterator for the argument iterator.
     * The newly constructed iterator will have a reference to the 
     * argument iterator.
     * Eqch time next() and prev() is called on the ReverseRandomIterator
     * the argument iterator is moved back and forward.
     * @param iter A RandomIterator.
     */
    public ReverseRandomIterator(RandomIterator iter) {
	iterator = iter;
    }

    /**
     * Create a copy of this. The copy can be moved without affcection this.
     * @return A new iterator pointing the same element as this.
     */
    public Iterator genericCopy()  {
	return new ReverseRandomIterator(Iter.copy(iterator));
    }

    /**
     * Assign this iterator to another iterator. This is faster than creating
     * a new ReverseRandomIterator object.
     */
    public void setTo(Iterator i) {
        ReverseRandomIterator ii = (ReverseRandomIterator)i;
	iterator = ii.iterator;
    }

    /**
     * Advance the iterator to the next element. Calls <code>prev()</code>
     * on the actual iterator.
     */
    public Iterator next()  {
	return iterator.prev();
    }

    /**
     * Advance the iterator to the previous element. Calls <code>next()</code>
     * on the actual iterator.
     */
    public Iterator prev()  {
	return iterator.next();
    }

    /**
     * Advance the iterator to the next element. Calls <code>prev(n)</code>
     * on the actual iterator.
     */
    public RandomIterator next(int n)  {
	return iterator.prev(n);
    }

    /**
     * Advance the iterator to the previous element. Calls <code>next(n)</code>
     * on the actual iterator.
     */
    public RandomIterator prev(int n)  {
	return iterator.next(n);
    }

    /**
     * Return the value pointed to by the iterator.
     */
    public Object get()  {
	iterator.prev();
	Object o = iterator.get();
	iterator.next();
	return o;
    }

    /**
     * Return the value pointed to by the iterator.
     */
    public Object get(int n)  {
	iterator.prev(n);
	Object o = iterator.get();
	iterator.next(n);
	return o;
    }

    /**
     * Places a new element in the position pointed to by the iterator.
     */
    public Object put(Object o)  {
	iterator.prev();
	iterator.put(o);
	iterator.next();
	return o;
    }
    /**
     * Places a new element in the position pointed to by the iterator.
     */
    public Object put(int n, Object o)  {
	iterator.prev(n);
	iterator.put(o);
	iterator.next(n);
	return o;
    }

    /**
     * Compares two iterators.
     * @return true if the iterators point to the same position.
     */
    public boolean cmp(Iterator i)  {
	return iterator.cmp(i);
    }

    /**
     * Compares two iterators for reletive order
     * @return true if this iterator is reachable from the argument iterator.
     */
    public boolean less(Iterator i)  {
	return ((ReverseRandomIterator)i).iterator.less(iterator);
    }

    /**
     * Returns the distance from this to the argument iterator.
     */
    public int distance(Iterator i) {
	return ((ReverseRandomIterator)i).iterator.distance(iterator);
    }
}
