package sjl;

/**
 * <code>Set</code> is a kind of associative container that supports 
 * unique keys (contains at most one of each key value) and provides 
 * for fast retrieval of the keys themselves.
 *
 * <p>Copyright <a href=..\Copyrght.htm>&copy</a> 1996 Finn Bock<p>
 */

public class Set implements AssociativeContainer, ReversibleContainer {
    RBtree tree;
    RBtreeIterator end;
    RBtreeIterator begin;
    RBtreeIterator tmpiterator;

    /**
     * Construct an empty set. 
     * @param predicate Use  predicate as the comparison object.
     */
    public Set(Predicate2 predicate) {
	tree = new RBtree(predicate, new Ident(), false);
	init();
    }

    /**
     * Construct a set and initialize it with the contents of another set. 
     * @param set Initialize the new set with the elements of set.
     */
    public Set(Set set) {
	tree = new RBtree(set.tree);
	init();
    }

    /**
     * Construct a set and initialize it with the elements in the 
     * range <code>[first,last)</code>.
     * @param first The beginning of the range.
     * @param last The end of the range.
     * @param predicate Use predicate as the comparison object.
     */
    public Set(InputIterator first, InputIterator last, Predicate2 predicate) {
	tree = new RBtree(predicate, new Ident(), false);
	init();
	for (InputIterator i = Iter.copy(first); !i.cmp(last); i.next())
	    tree.insert(i.get());
    }

    /**
     * Erase all the elements in the container. 
     */
    public void flush() {
	tree = new RBtree(tree.predicate, new Ident(), false);
	init();
    }

    /**
     * Compare the elements in this container with the elements 
     * in another container.
     * @return <code>true</code> is the elements match.
     */
    public boolean equals(Object container) {
	if ((container != null) && (container instanceof Container)) {
	    Container c = (Container)container;
	    return size() == c.size() &&  Algo.equal(beginRef(), endRef(),
			(InputIterator)c.beginGeneric());
	}
	return false;
    }

    /**
     * Returns the iterator that represents the beginning of the set. 
     */
    public BidirectionalIterator begin() {
	begin.node = tree.begin();
	return new RBtreeIterator(begin);
    }

    /**
     * Returns a reference to the iterator that represents the beginning 
     * of the set. 
     */
    public BidirectionalIterator beginRef() {
	begin.node = tree.begin();
	return begin;
    }

    /**
     * Returns the iterator that represents the end of the set. 
     */
    public BidirectionalIterator end() {
	return new RBtreeIterator(end);
    }

    /**
     * Returns a reference to the iterator that represents the end of the set. 
     */
    public BidirectionalIterator endRef() {
	return end;
    }

    /**
     * Returns the iterator that represents the beginning of the set as an
     * untyped iterator. 
     */
    public ForwardIterator beginGeneric() {
	begin.node = tree.begin();
	return begin;
    }

    /**
     * Returns the iterator that represents the end of the set as an
     * untyped iterator. 
     */
    public ForwardIterator endGeneric()  {
	return end;
    }

    /**
     * Returns a copy of the reverse iterator that represents the 
     * beginning (end) of the list.
     */
    public ReverseBidirectionalIterator rbegin() {
	return new ReverseBidirectionalIterator(end);
    }

    /**
     * Returns a copy of the iterator that represents the end (beginning)
     * of the list.
     */
    public ReverseBidirectionalIterator rend() {
	return new ReverseBidirectionalIterator(begin);
    }

    /**
     * Returns a copy of the reverse iterator that represents the 
     * beginning (end) of the list as an untyped Iterator.
     */
    public Iterator rbeginGeneric() {
	return rbegin();
    }

    /**
     * Returns a copy of the iterator that represents the end (beginning)
     * of the list as an untyped Iterator.
     */
    public Iterator rendGeneric() {
	return rend();
    }

    /**
     * Returns the number of elements in the set.
     */
    public int size() {
	return tree.size();
    }

    /**
     * Returns the maximum number of elements allowed in a set.
     */
    public int max_size() {
	return Integer.MAX_VALUE;
    }

    /**
     * Returns <code>true</code> if the set does not contain any elements.
     */
    public boolean empty() {
	return tree.size() == 0;
    }

    /**
     * Insert an element into the set. The element is only inserted if 
     * there is no element in the container with a key equal to the key 
     * of value.
     * @param value The element to insert.
     * @return The first component of pair is an iterator that point to the 
     * element with a key equal to the key of value. The second component of 
     * pair is a Boolean that is true if value was actually inserted, and 
     * false if the container already contained a element equal to value.
     */
    public Pair insert(Object value) {
	Pair p = tree.insert(value);
	tmpiterator.node = (RBtreeNode)p.first;
	p.first = tmpiterator;
	return p;
    }

    /**
     * Insert an element into the set. The element is only inserted if 
     * there is no element in the container with a key equal to the key 
     * of value.
     * @param position A hint pointing to where the insert should start to
     *        search. If the hint is correct, and the value is inserted 
     *        right after position, the time is amortized constant.
     * @param value The element to insert.
     * @return The an iterator that point to the element with a key equal 
     * to the key of value
     */
    public RBtreeIterator insert(Iterator position, Object value) {
	tmpiterator.node = tree.insert(((RBtreeIterator)position).node, value);
	return tmpiterator;
    }

    /**
     * Insert the elements from the range <code>[first,last)</code> into 
     * the set.
     * @param first The beginning of the range.
     * @param last The end of the range.
     */
    public void insert(InputIterator first, InputIterator last) {
	for (InputIterator i = Iter.copy(first); !i.cmp(last); i.next()) 
	    tree.insert(i.get());
    }

    /**
     * Erase the element pointed to by the iterator.
     * @param position Points to the element to be removed.
     */
    public void erase(Iterator position) {
	tree.erase(((RBtreeIterator)position).node);
    }

    /**
     * Erase the element with key equal to value.
     * @return The number of element erased.
     */
    public int erase(Object value) {
	return tree.erase(value);
    }

    /**
     * Erase the elements in the range <code>[first,last)</code>.
     * @param first The beginning of the range.
     * @param last The end of the range.
     */
    public void erase(Iterator first, Iterator last) {
	tree.erase(((RBtreeIterator)first).node, ((RBtreeIterator)last).node);
    }

    /**
     * Search the set for an element with a key equal to value.
     * @param value The key to search for.
     * @return An Iterator pointing to the element with a key equal to value,
     *          or end() if such an element is not found.
     */
    public ForwardIterator find(Object value) {
	tmpiterator.node = tree.find(value);
	return tmpiterator;
    }

    /**
     * Returns the number of element with a key equal to value.
     */
    public int count(Object value) {
	return tree.count(value);
    }

    /**
     * Returns an iterator pointing to the first element with a key not
     * less than value.
     */
    public RBtreeIterator lower_bound(Object value) {
	tmpiterator.node = tree.lower_bound(value);
	return tmpiterator;
    }

    /**
     * Returns an iterator pointing to the first element with a key greter
     * than value.
     */
    public RBtreeIterator upper_bound(Object value) {
	tmpiterator.node = tree.upper_bound(value);
	return tmpiterator;
    }

    /**
     * Returns a pair like 
     * <code>Pair(lower_bound(value), upper_bound(value))</code>.
     */
    public Pair equal_range(Object value) {
	return tree.equal_range(value);
    }

    private void init() {
	begin = new RBtreeIterator(tree, tree.begin());
	end = new RBtreeIterator(tree, tree.end());
	tmpiterator = new RBtreeIterator(tree, null);
    }

    public void dump() {
	tree.dump();
    }
}
