package sjl;

/**
 * <code>SingleList</code> is a kind of sequence that supports forward iterators
 * and allow constant time insert and erase operations anywhere within the 
 * sequense, with storage management handled automatically. Unlike vectors 
 * and deques, fast random access to list elements are not suported, 
 * but many algorithms only need sequential access anyway. <p>
 * 
 * <code>insert</code> does not affect the validity of iterators. 
 * Insertion of a single element into a list takes constant time. 
 * Insertion of multiple elements into a list is linear in the number of 
 * elements inserted. <p>
 * 
 * <code>erase</code> invalidates only the iterators to the erased elements. 
 * Erasing a single element is constant time. 
 * Erasing a range in a list is linear time in the size of the range.
 *
 * <p>Copyright <a href=..\Copyrght.htm>&copy</a> 1996 Finn Bock<p>
 */

public class SingleList implements FrontInsertContainer {
    SingleListNode head;
    int length;
    SingleListIterator begin;
    SingleListIterator tmpIter;
    SingleListIterator end;

    /**
     * Constructs an empty list.
     */
    public SingleList() {
	length = 0;
	head = get_node();
	head.next = null;
	begin = new SingleListIterator(head);
	end = new SingleListIterator(head);
    }

    /**
     * Constructs a list with <code>n</code> element each pointing to value.
     * @param n The number of elements the list will contain.
     * @param value The object that is inserted in each in the list.
     */
    public SingleList(int n, Object value) {
	this();
	insert(begin, n, value);
    }

    /**
     * Constructs a list with elements found in the range 
     * <code>[first,last)</code>.
     * @param first The beginning of the range.
     * @param last The end of the range.
     */
    public SingleList(InputIterator first, InputIterator last) {
	this();
	insert(begin, first, last);
    }

    /**
     * Constructs a list with all the elements from another list.
     * @param list The elements in this list is copied into the newly 
     * contructed list.
     */
    public SingleList(SingleList list) {
	this();
	insert(begin, list.begin(), list.end());
    }

    /** 
     * Erase all the elements in the container.
     */
    public void flush() {
	length = 0;
	head = get_node();
	head.next = null;
	begin._set(head);
	end._set(head);
    }

    /**
     * Swaps the contents of the current list with those of the input list.
     * The current list replaces x and vice versa.
     */
    public void swap(SingleList x) {
	SingleListNode p1 = head; head = x.head; x.head = p1;
	int p2 = length; length = x.length; x.length = p2;
        SingleListIterator p3 = begin; begin = x.begin; x.begin = p3;
        SingleListIterator p4 = end; end = x.end; x.end = p4;
        SingleListIterator p5 = tmpIter; tmpIter = x.tmpIter; x.tmpIter = p3;
    }

    /**
     * Compare the elements in this container with the elements 
     * in another container.
     * @return <code>true</code> is the elements match.
     */
    public boolean equals(Object container) {
	if ((container != null) && (container instanceof Container)) {
	    Container c = (Container)container;
	    return size() == c.size() &&  Algo.equal(beginRef(), endRef(),
			(InputIterator)c.beginGeneric());
	}
	return false;
    }

    /**
     * Returns the iterator that represents the beginning of the list.
     */
    public ForwardIterator begin() {
	return new SingleListIterator(head);
    }

    /**
     * Returns a reference to the iterator that represents the 
     * beginning of the list.
     */
    public ForwardIterator beginRef() {
	return begin;
    }

    /**
     * Returns the iterator that represents the end of the list.
     */
    public ForwardIterator end() {
	return new SingleListIterator(end);
    }

    /**
     * Returns a reference to the iterator that represents the end of the list.
     */
    public ForwardIterator endRef() {
	return end;
    }

    /**
     * Returns the iterator that represents the beginning of the list as a
     * untyped iterator.
     */
    public ForwardIterator beginGeneric() {
	return begin;
    }

    /**
     * Returns the iterator that represents the end of the list as a
     * untyped iterator.
     */
    public ForwardIterator endGeneric() {
	return end;
    }

    /**
     * Returns the first element in the list.
     */
    public Object front() {
	return head.next.data;
    }

    /**
     * Returns the last element in the list.
     */
    public Object back() {
	return end.node.data;
    }


    /**
     * Returns the number of elements stored in the list.
     */
    public int size() {
	return length;
    }

    /**
     * Returns the maximum number of elements allowed in the list. 
     */
    public int max_size() {
	return Integer.MAX_VALUE;
    }

    /**
     * Returns <code>true</code> if the list does not contain any elements.
     */
    public boolean empty() {
	return length == 0;
    }

    /**
     * Insert at the beginning of the list.
     * @param value the element to insert. 
     */
    public void push_front(Object value) {
        insert(beginRef(), value);
    }

    /**
     * Insert at the end of the list.
     * @param value the element to insert. 
     */
    public void push_back(Object value) {
	insert(endRef(), value);
    }

    /**
     * Insert the element at the specified position.
     * @param position An iterator that points to the position where 
     *                 the element is inserted. 
     * @param value The element.
     */
    public Iterator insert(Iterator position, Object value) {
	SingleListIterator p = (SingleListIterator)position;
	SingleListNode tmp =  get_node();
        tmp.data = value;
        tmp.next = p.node.next;
	p.node.next = tmp;
        length++;
	if (p.node == end.node)
	    end.node = tmp;
	return new SingleListIterator(p);
    }

    /**
     * Insert <code>n</code> copy of the element at the specified position.
     * @param position An iterator that points to the position where 
     *                 the elements are inserted. 
     * @param n The number of elements.
     * @param value The element.
     */
    public void insert(Iterator position, int n, Object value) {
	while (n-- > 0) 
	    insert(position, value);
    }

    /**
     * Insert the element in the range <code>[first,last)</code> into the list 
     * at the specified position.
     * @param position An iterator that points to the position where 
     *                 the elements are inserted. 
     * @param first The beginning of the range
     * @param last The end of the range
     */
    public void insert(Iterator position, InputIterator first, InputIterator last) {
	ForwardIterator p = Iter.copy((ForwardIterator)position);
	first = Iter.copy(first);
	while (!first.cmp(last)) {
	    insert(p, first.get());
	    p.next();
	    first.next();
	}
    }

    /**
     * Remove the first element in the list.
     */
    public void pop_front() {
	erase(beginRef());
    }
    
    /**
     * Remove the element to which the iterator points.
     * @param position the element to which the iterator point is removed.
     */
    public void erase(Iterator position) {
	SingleListIterator p = (SingleListIterator)position;
	p.node.next = p.node.next.next;
	if (p.node.next == null)
	    end.node = p.node;
	length--;
    }

    /**
     * Remove the elements in the range <code>[first,last)</code> from the list.
     * @param first The beginning of the range.
     * @param last The end of the range.
     */
    public void erase(Iterator first, Iterator last) {
	SingleListIterator f = ((SingleListIterator)first).copy();
	while (!f.cmp(last)) {
	    erase(first);
	    f.next();
	}
    }

    /**
     * Insert the contents of another list into this list before position.
     * All elements are removed from the other list. Essentially the 
     * contents of the orther list is transferred in into this list.
     * @param position The elements are inserted before this position.
     * @param x The source list.
     */
    public void splice(Iterator position, SingleList x) {
	if (!x.empty()) {
	    transfer(position, x, x.begin(), x.end());
	    length += x.length;
	    x.length = 0;
	}
    }

    /**
     * Insert the element pointed to by the iterator <code>i</code> from
     * another list into this list before a position.
     * The element is removed from the other list. 
     * @param position The elements are inserted before this position.
     * @param x The source list.
     * @param i The source element.
     */
    public void splice(Iterator position, SingleList x, Iterator i) {
	Iterator j = Iter.copy((SingleListIterator)i);
	if (position.cmp(i) || position.cmp(j.next()))
	    return;
	transfer(position, x, i, j);
	++length;
	--x.length;
    }

    /**
     * Insert elements in the range into this list and remove the 
     * elements from the other list.
     * Elements in the range <code>[first, last)</code> is inserted into 
     * this list before position. The elements are removed from the other list.
     * @param position The elements are inserted before this position.
     * @param x The source list.
     * @param first The beginning of the range.
     * @param last The end of the range.
     */
    public void splice(Iterator position, SingleList x, Iterator first, Iterator last) {
	if (!first.cmp(last)) {
	    if (x != this) {
		int n = ((SingleListIterator)first).distance(last);
		x.length -= n;
		length += n;
	    }
	    transfer(position, x, first, last);
	}
    }

    /**
     * Insert elements from another list into this list and remove the
     * elements from the other list.
     * @param x The source list.
     */
    public void splice(SingleList x) {
	splice(begin(), x, x.begin(), x.end());
    }

    /**
     * Insert an element from another list into this list and remove the
     * element from the other list.
     * @param x The source list.
     * @param position The element which will be inserted.
     */
    public void splice(SingleList x, Iterator position) {
	ForwardIterator j = Iter.copy((ForwardIterator)position);
	splice(begin(), x, position, j.next());
    }

    /**
     * Remove all elements in the list where the element is equals to the
     * value. Remove is stable, that is, the relative order of the 
     * elements that not removed is the same as their relative order in 
     * the original list.
     * @param value The value that the elements are compared with.
     */
    public void remove(Object value) {
	ForwardIterator first = begin();
	ForwardIterator last = end();
	while (!first.cmp(last)) {
	    if (first.get().equals(value)) {
		ForwardIterator tmp = Iter.copy(first);
		first.next();
		erase(tmp);
	    } else
		first.next();
	}
    }
		
    /**
     * Unique erases all but the first element from every 
     * consecutive group of equal element in the list.
     * value. Remove is stable, that is, the relative order of the 
     * elements that not removed is the same as their relative order in 
     * the original list.
     * @param value The value that the elements are compared with.
     */
    public void unique() {
	ForwardIterator first = begin();
	ForwardIterator last = end();
	if (first.cmp(last))
	    return;
	ForwardIterator next = Iter.copy(first);
	while (!next.next().cmp(last)) {
	    if (first.get().equals(next.get())) 
		erase(next);
	    else
		first.setTo(next);
	    next.setTo(first);
	}
    }

    /**
     * Merge merges the argument list into this list 
     * (both are assumed to be sorted).
     * The merge is stable, that is, for equal elements in the two lists,
     * the elements from this list always precede elements from the
     * argument list. The argument list is empty after the merge.
     * @param x The list that is merge into this list.
     */
    public void merge(SingleList x, Predicate2 pred) {
	ForwardIterator first1 = begin();
	ForwardIterator last1 = end();
	ForwardIterator first2 = x.begin();
	ForwardIterator last2 = x.end();
	ForwardIterator next = Iter.copy(first2);

	while (!first1.cmp(last1) && !first2.cmp(last2)) {
	    if (pred.compare(first2.get(), first1.get())) {
		next.setTo(first2);
		transfer(first1, x, first2, next.next());
		first2.setTo(next);
	    } else
		first1.next();
	}
	if (!first2.cmp(last2))
	    transfer(last1, x, first2, last2);
	length += x.length;
	x.length = 0;
    }

    /**
     * Reverse reverses the oerder of the element in the list.
     */
    public void reverse() {
	if (size() < 2)
	    return;
	ForwardIterator first = begin();
	for (first.next(); !first.cmp(end); ) {
	    Iterator old = Iter.copy(first);
	    first.next();
	    transfer(begin(), this, old, first);
	}
    }

    /**
     * Sort sorts the list according to the compare function object. It is
     * stable, that is, the relative order of the equal elements is preserved.
     */
    public void sort(Predicate2 pred) {
	if (size() < 2)
	    return;
	SingleList carry = new SingleList();
	SingleList counter[] = new SingleList[64];
	for (int i = 0; i < 64; i++)
	    counter[i] = new SingleList();
	int fill = 0;
	while (!empty()) {
	    carry.splice(carry.begin(), this, begin());
	    int i = 0;
	    while (i < fill && !counter[i].empty()) {
		counter[i].merge(carry, pred);
	 	carry.swap(counter[i++]);
	    }
	    carry.swap(counter[i]);
	    if (i == fill)
		++fill;
	}
	for (int i = 1; i < fill; i++)
	    counter[i].merge(counter[i-1], pred);
	swap(counter[fill - 1]);
    }

    private void transfer(Iterator position, SingleList x, Iterator first, Iterator last) {
	SingleListIterator l = (SingleListIterator)last;
	SingleListIterator f = (SingleListIterator)first;
	SingleListIterator p = (SingleListIterator)position;
	l.node.next = p.node.next;
	p.node.next = f.node.next;
	f.node.next = null;
	if (end.node == p.node)
	    end.node = l.node;
	if (x.end.node == l.node)
	    x.end.node = f.node;
    }

    private SingleListNode get_node() {
	return new SingleListNode();
    }
}
