package sjl;

/**
 * <code>Vector</code> is a kind of sequence that supports random access 
 * iterators.
 * In additition it supports (amortized) constant time insert and erase 
 * operations at the end; insert and erase in the middle take linear time. 
 * Storage management is handled automatically.<p>
 * 
 * <code>insert</code> causes reallocation if the new size is greater 
 * than the old capacity. 
 * The iterators before the insertion point always remain valid.
 * Inserting a single element into a vector is linear in the distance from the
 * insertion point to the end of the vector. The amortized complexity over 
 * the lifetime of a vector of inserting a single element at the end is 
 * constant. Insertion of multiple elements into a vector with a single 
 * call of the insert method is linear in the sum of the number of elements 
 * plus the distance to the end of the vector. <br>
 * In other words, it is mush faster to insert many elements into the middle 
 * of a vector at once, than to do the insertion one at a time. <p>
 * 
 * <code>erase</code> invalidates all the iterators after the point of 
 * the erase.
 *
 * <p>Copyright <a href=..\Copyrght.htm>&copy</a> 1996 Finn Bock<p>
 */ 

public class Vector implements ReversibleContainer, BackInsertContainer {
    /**
     * Private. The actual data array.
     */
    protected Object data[];
    private VectorIterator start;
    private VectorIterator finish;
    private VectorIterator end_storage;

    /**
     * Construct a new empty Vector.
     */
    public Vector() {
	data = new Object[10];
	start = new VectorIterator(this, 0);
	finish = new VectorIterator(this, 0);
	end_storage = new VectorIterator(this, 10);
    }

    /**
     * Construct a new Vector with an initial size of <code>size</code> 
     * elements, each element initialized to with a reference to value.
     * @param size The size of the new vector.
     * @param value The initial value of each element in the vector.
     */
    public Vector(int size, Object value) {
	data = new Object[size];
	start = new VectorIterator(this, 0);
	finish = new VectorIterator(this, size);
	end_storage = new VectorIterator(this, size);
	Algo.fill(beginRef(), beginRef(), value);
    }

    /**
     * Construct a new Vector with the size and contents of another vector.
     * @param vector The vector which is copied into the new vector.
     */
    public Vector(Vector vector) {
	int size = vector.size();
	data = new Object[size];
	start = new VectorIterator(this, 0);
	finish = new VectorIterator(this, size);
	end_storage = new VectorIterator(this, size);
	Algo.copy(vector.beginRef(), vector.endRef(), start);
    }

    /**
     * Construct a new Vector with the contents of the range 
     * <code>[first,last)</code>.
     * @param first The beginning of the range.
     * @param last The end of the range.
     */
    public Vector(ForwardIterator first, ForwardIterator last) {
	int size = first.distance(last);
	data = new Object[size];
	start = new VectorIterator(this, 0);
	finish = new VectorIterator(this, size);
	end_storage = new VectorIterator(this, size);
	Algo.copy(first, last, beginRef());
    }

    /**
     * Erase all the elements in the container. 
     */
    public void flush() {
	data = new Object[10];
	start = new VectorIterator(this, 0);
	finish = new VectorIterator(this, 0);
	end_storage = new VectorIterator(this, 10);
    }

    /**
     * Compare the elements in this container with the elements 
     * in another container.
     * @return <code>true</code> is the elements match.
     */
    public boolean equals(Object container) {
	if ((container != null) && (container instanceof Container)) {
	    Container c = (Container)container;
	    return size() == c.size() &&  Algo.equal(beginRef(), endRef(),
			(InputIterator)c.beginGeneric());
	}
	return false;
    }

    /**
     * Returns the iterator that represents the beginning of the vector.
     */
    public VectorIterator begin() {
	return new VectorIterator(start);
    }

    /**
     * Returns a reference to the iterator that represents the beginning of 
     * the vector.
     */
    public VectorIterator beginRef() {
	return start;
    }

    /**
     * Returns the iterator that represents the end of the vector.
     */
    public VectorIterator end() {
	return new VectorIterator(finish);
    }

    /**
     * Returns a reference to the iterator that represents the end of 
     * the vector.
     */
    public VectorIterator endRef() {
	return finish;
    }

    /**
     * Returns the iterator that represents the beginning of the vector as
     * an untyped iterator.
     */
    public ForwardIterator beginGeneric() {
	return start;
    }

    /**
     * Returns the iterator that represents the end of the vector as
     * an untyped iterator.
     */
    public ForwardIterator endGeneric() {
	return finish;
    }

    /**
     * Returns a copy of the reverse iterator that represents the 
     * beginning (end) of the list.
     */
    public ReverseRandomIterator rbegin() {
	return new ReverseRandomIterator(finish);
    }

    /**
     * Returns a copy of the iterator that represents the end (beginning)
     * of the list.
     */
    public ReverseRandomIterator rend() {
	return new ReverseRandomIterator(start);
    }

    /**
     * Returns a copy of the reverse iterator that represents the 
     * beginning (end) of the list as an untyped Iterator.
     */
    public Iterator rbeginGeneric() {
	return rbegin();
    }

    /**
     * Returns a copy of the iterator that represents the end (beginning)
     * of the list as an untyped Iterator.
     */
    public Iterator rendGeneric() {
	return rend();
    }

    /**
     * Returns the number of elements stored in the vector.
     */
    public int size() {
	return finish.pos - start.pos;
    }

    /**
     * Returns the maximum number of elements that can be stored in a vector.
     */
    public int max_size() {
	return Integer.MAX_VALUE;
    }

    /**
     * Returns the maximum number of elements that can be stored in the vector 
     * without reallocation.
     */
    public int capacity() {
	return data.length;
    }

    /**
     * Returns <code>true</code> if the vector does not contain any elements.
     */
    public boolean empty() {
	return start.pos == finish.pos;
    }

    /**
     * Returns the <code>n</code>th element in the container.
     */
    public Object get(int n) {
	return data[n];
    }

    /**
     * Set the <code>n</code>th element in the container to o.
     */
    public Object put(int n, Object o) {
	return data[n] = o;
    }

    /**
     * Return the first element in the container.
     */
    public Object front() {
	return start.get();
    }

    /**
     * Return the last element in the container.
     */
    public Object back() {
	finish.prev();
        Object o = finish.get();
	finish.next();
	return o;
    }

    /**
     * Add an element to the end of the vector.
     */
    public void push_back(Object o) {
	if (!finish.cmp(end_storage)) {
	    finish.put(o);
	    finish.next();
	}
	else 
	    insert_aux(finish, o);
    }

    /**
     * Insert an element into the vector at the specified position.
     * Elements located after the position are moved.
     * @param position insert the element at this position.
     * @param o the element to insert.
     * @return An iterator to the position where the element was inserted.
     */
    public Iterator insert(Iterator position, Object o) {
	int pos = ((VectorIterator)position).pos();

	if (!finish.cmp(end_storage) && position.cmp(finish)) {
            data[pos] = o;
	    finish.next();
	}
	else 
 	    insert_aux(position, o);
	return position;
    }

    /**
     * Insert n references to element into the vector at the specified position.
     * Elements located after the position are moved.
     * @param position insert the element at this position.
     * @param n the number of elements to insert.
     * @param o the element to insert.
     */
    public void insert(Iterator position, int n, Object o) {
	VectorIterator p = (VectorIterator)position;
	if (n == 0)
	    return;
	int pos = p.pos();

	if (finish.distance(end_storage) >= n) {
	    System.arraycopy(data, pos, data, pos + n, finish.pos() - pos);
	    Algo.fill(p, n, o);
	}
	else {
	    Object olddata[] = data;
	    data = new Object[olddata.length + n];
	    System.arraycopy(olddata, 0, data, 0, pos);
	    Algo.fill(p, n, o);
	    System.arraycopy(olddata, pos, data, pos + n, olddata.length - pos);
	    end_storage.pos(data.length);
	}
	finish.next(n);
    }

    /**
     * The elements in the range <code>[first,last)</code> are inserted 
     * into vector at the
     * specified position.
     * Elements located after the position are moved.
     * @param position insert the element at this position.
     * @param first the beginning of the range.
     * @param last the end of the range.
     */
    public void insert(Iterator position, InputIterator first, InputIterator last) {
	VectorIterator p = (VectorIterator)position;
	if (first.cmp(last))
	    return;
	int n = ((ForwardIterator)first).distance(last);
	int pos = p.pos();

	if (finish.distance(end_storage) >= n) {
	    System.arraycopy(data, pos, data, pos + n, n);
	    Algo.copy(first, last, p);
	}
	else {
	    Object olddata[] = data;
	    Object data[] = new Object[olddata.length + n];
	    System.arraycopy(olddata, 0, data, 0, pos);
	    Algo.copy(first, last, p);
	    System.arraycopy(olddata, pos, data, pos + n, olddata.length - pos);
	    end_storage.pos(data.length);
	}
	finish.next();
    }

    /**
     * Removes the last element in vector.
     */
    public void pop_back() {
	finish.prev();
	finish.put(null);
    }

    /**
     * Removes the element specified by position.
     */
    public void erase(Iterator position) {
	VectorIterator p = (VectorIterator)position;
	VectorIterator pos = (VectorIterator)Iter.copy(p).next();
	if (!finish.cmp(pos))
	    Algo.copy(pos, finish, p);
	finish.prev();
	finish.put(null);
    }

    /**
     * Removes the elements in the specified range.
     * @param first the beginning of the range.
     * @param last the end of the range.
     */
    public void erase(Iterator first, Iterator last) {
	VectorIterator f = (VectorIterator)first;
	VectorIterator l = (VectorIterator)last;
        
	VectorIterator i = (VectorIterator)Algo.copy(l, finish, f);
	Algo.fill(i, finish, null);
	finish.prev(f.distance(l));
    }

    private void insert_aux(Iterator position, Object o) {
	int pos = ((VectorIterator)position).pos();

	if (!finish.cmp(end_storage)) {
	    System.arraycopy(data, pos, data, pos + 1, finish.pos() - pos);
	    data[pos] = o;
	}
	else {
	    Object olddata[] = data;
	    data = new Object[olddata.length * 2];
	    System.arraycopy(olddata, 0, data, 0, pos);
	    data[pos] = o;
	    System.arraycopy(olddata, pos, data, pos + 1, finish.pos() - pos);
	    end_storage.pos(data.length);
	}
	finish.next();
    }

    /**
     * Returns the string representation of this vector.
     */
    public String toString()  {
        StringBuffer buf = new StringBuffer();
        buf.append(start);
        buf.append(finish);
        buf.append(end_storage);
        return buf.toString();
    }
}
